# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: fx/dex/v1/query.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.api import annotations_pb2 as google_dot_api_dot_annotations__pb2
from fxsdk.x.fx.dex.v1 import order_pb2 as fx_dot_dex_dot_v1_dot_order__pb2
from fxsdk.x.fx.dex.v1 import match_pb2 as fx_dot_dex_dot_v1_dot_match__pb2
from fxsdk.x.fx.dex.v1 import position_pb2 as fx_dot_dex_dot_v1_dot_position__pb2
from fxsdk.x.fx.dex.v1 import params_pb2 as fx_dot_dex_dot_v1_dot_params__pb2
from fxsdk.x.fx.dex.v1 import funding_pb2 as fx_dot_dex_dot_v1_dot_funding__pb2
from fxsdk.x.gogoproto import gogo_pb2 as gogoproto_dot_gogo__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x15\x66x/dex/v1/query.proto\x12\tfx.dex.v1\x1a\x1cgoogle/api/annotations.proto\x1a\x15\x66x/dex/v1/order.proto\x1a\x15\x66x/dex/v1/match.proto\x1a\x18\x66x/dex/v1/position.proto\x1a\x16\x66x/dex/v1/params.proto\x1a\x17\x66x/dex/v1/funding.proto\x1a\x14gogoproto/gogo.proto\"\x11\n\x0fQueryReserveReq\"7\n\x10QueryReserveResp\x12#\n\x07reserve\x18\x01 \x01(\x0b\x32\x12.fx.dex.v1.Reserve\"\x13\n\x11QueryIsFundingReq\"(\n\x12QueryIsFundingResp\x12\x12\n\nis_funding\x18\x01 \x01(\x08\"\x18\n\x16QueryStoreStatisticReq\"d\n\x17QueryStoreStatisticResp\x12\x17\n\x0fstore_order_num\x18\x01 \x01(\x03\x12\x16\n\x0eorderbook_size\x18\x02 \x01(\x03\x12\x18\n\x10orderbook_length\x18\x03 \x01(\x03\"\x1d\n\nQueryMAReq\x12\x0f\n\x07pair_id\x18\x01 \x01(\t\"~\n\x0bQueryMAResp\x12:\n\raverage_price\x18\x01 \x03(\tB#\xc8\xde\x1f\x00\xda\xde\x1f\x1b\x63osmossdk.io/math.LegacyDec\x12\x33\n\x06price2\x18\x02 \x01(\tB#\xc8\xde\x1f\x00\xda\xde\x1f\x1b\x63osmossdk.io/math.LegacyDec\"@\n\x1aQueryMarkAndOraclePriceReq\x12\x0f\n\x07pair_id\x18\x01 \x01(\t\x12\x11\n\tquery_all\x18\x02 \x01(\x08\"R\n\x1bQueryMarkAndOraclePriceResp\x12\x33\n\x06prices\x18\x01 \x03(\x0b\x32\x1d.fx.dex.v1.MarkAndOraclePriceB\x04\xc8\xde\x1f\x00\"\x10\n\x0eQueryParamsReq\":\n\x0fQueryParamsResp\x12\'\n\x06params\x18\x01 \x01(\x0b\x32\x11.fx.dex.v1.ParamsB\x04\xc8\xde\x1f\x00\"\x91\x01\n\x12QueryOrdersRequest\x12\r\n\x05owner\x18\x01 \x01(\t\x12\x0f\n\x07pair_id\x18\x02 \x01(\t\x12,\n\x04page\x18\x03 \x01(\tB\x1e\xc8\xde\x1f\x00\xda\xde\x1f\x16\x63osmossdk.io/math.Uint\x12-\n\x05limit\x18\x04 \x01(\tB\x1e\xc8\xde\x1f\x00\xda\xde\x1f\x16\x63osmossdk.io/math.Uint\"=\n\x13QueryOrdersResponse\x12&\n\x06orders\x18\x01 \x03(\x0b\x32\x10.fx.dex.v1.OrderB\x04\xc8\xde\x1f\x00\"%\n\x11QueryOrderRequest\x12\x10\n\x08order_id\x18\x01 \x01(\t\";\n\x12QueryOrderResponse\x12%\n\x05order\x18\x01 \x01(\x0b\x32\x10.fx.dex.v1.OrderB\x04\xc8\xde\x1f\x00\"$\n\x11QueryOrderbookReq\x12\x0f\n\x07pair_id\x18\x01 \x01(\t\"j\n\x12QueryOrderbookResp\x12)\n\x04\x41sks\x18\x01 \x03(\x0b\x32\x15.fx.dex.v1.OrderDepthB\x04\xc8\xde\x1f\x00\x12)\n\x04\x42ids\x18\x02 \x03(\x0b\x32\x15.fx.dex.v1.OrderDepthB\x04\xc8\xde\x1f\x00\"\xbc\x01\n\x11QueryPositionsReq\x12\r\n\x05owner\x18\x01 \x01(\t\x12*\n\tdirection\x18\x02 \x01(\x0e\x32\x17.fx.dex.v1.PosDirection\x12\x0f\n\x07pair_id\x18\x03 \x01(\t\x12,\n\x04page\x18\x04 \x01(\tB\x1e\xc8\xde\x1f\x00\xda\xde\x1f\x16\x63osmossdk.io/math.Uint\x12-\n\x05limit\x18\x05 \x01(\tB\x1e\xc8\xde\x1f\x00\xda\xde\x1f\x16\x63osmossdk.io/math.Uint\"B\n\x12QueryPositionsResp\x12,\n\tpositions\x18\x01 \x03(\x0b\x32\x13.fx.dex.v1.PositionB\x04\xc8\xde\x1f\x00\"2\n\x10QueryPositionReq\x12\r\n\x05owner\x18\x01 \x01(\t\x12\x0f\n\x07pair_id\x18\x02 \x01(\t\"A\n\x11QueryPositionResp\x12,\n\tpositions\x18\x01 \x03(\x0b\x32\x13.fx.dex.v1.PositionB\x04\xc8\xde\x1f\x00\"\x11\n\x0fQueryFundingReq\"=\n\x10QueryFundingResp\x12)\n\x07\x66unding\x18\x01 \x01(\x0b\x32\x12.fx.dex.v1.FundingB\x04\xc8\xde\x1f\x00\"4\n\x18QueryPairFundingRatesReq\x12\x18\n\x10last_or_realtime\x18\x01 \x01(\x08\"Y\n\x19QueryPairFundingRatesResp\x12<\n\x12pair_funding_rates\x18\x01 \x03(\x0b\x32\x1a.fx.dex.v1.PairFundingRateB\x04\xc8\xde\x1f\x00\"\x15\n\x13QueryFundingTimeReq\"J\n\x14QueryFundingTimeResp\x12\x32\n\x0c\x66unding_time\x18\x01 \x01(\x0b\x32\x16.fx.dex.v1.FundingTimeB\x04\xc8\xde\x1f\x00\"P\n\tPairPrice\x12\x0f\n\x07pair_id\x18\x01 \x01(\t\x12\x32\n\x05price\x18\x02 \x01(\tB#\xc8\xde\x1f\x00\xda\xde\x1f\x1b\x63osmossdk.io/math.LegacyDec\"7\n\x11QueryDealPriceReq\x12\x0f\n\x07pair_id\x18\x01 \x01(\t\x12\x11\n\tquery_all\x18\x02 \x01(\x08\"D\n\x12QueryDealPriceResp\x12.\n\npair_price\x18\x01 \x03(\x0b\x32\x14.fx.dex.v1.PairPriceB\x04\xc8\xde\x1f\x00\"<\n\x13QueryMatchResultReq\x12\x0f\n\x07pair_id\x18\x01 \x01(\t\x12\x14\n\x0c\x62lock_number\x18\x02 \x01(\x03\"J\n\x14QueryMatchResultResp\x12\x32\n\x0cmatch_result\x18\x01 \x01(\x0b\x32\x16.fx.dex.v1.MatchResultB\x04\xc8\xde\x1f\x00\"7\n\x11QueryMarkPriceReq\x12\x0f\n\x07pair_id\x18\x01 \x01(\t\x12\x11\n\tquery_all\x18\x02 \x01(\x08\"I\n\x12QueryMarkPriceResp\x12\x33\n\x0fpair_mark_price\x18\x01 \x03(\x0b\x32\x14.fx.dex.v1.PairPriceB\x04\xc8\xde\x1f\x00\"\x99\x01\n\x12MarkAndOraclePrice\x12\x0f\n\x07pair_id\x18\x01 \x01(\t\x12\x37\n\nmark_price\x18\x02 \x01(\tB#\xc8\xde\x1f\x00\xda\xde\x1f\x1b\x63osmossdk.io/math.LegacyDec\x12\x39\n\x0coracle_price\x18\x03 \x01(\tB#\xc8\xde\x1f\x00\xda\xde\x1f\x1b\x63osmossdk.io/math.LegacyDec\"\x97\x02\n\x18QueryLiquidationPriceReq\x12\x0f\n\x07pair_id\x18\x01 \x01(\t\x12\x10\n\x08leverage\x18\x02 \x01(\x04\x12*\n\tdirection\x18\x03 \x01(\x0e\x32\x17.fx.dex.v1.PosDirection\x12\x38\n\x0b\x65ntry_price\x18\x04 \x01(\tB#\xc8\xde\x1f\x00\xda\xde\x1f\x1b\x63osmossdk.io/math.LegacyDec\x12\x35\n\x08quantity\x18\x05 \x01(\tB#\xc8\xde\x1f\x00\xda\xde\x1f\x1b\x63osmossdk.io/math.LegacyDec\x12;\n\x0ewallet_balance\x18\x06 \x01(\tB#\xc8\xde\x1f\x00\xda\xde\x1f\x1b\x63osmossdk.io/math.LegacyDec\"[\n\x19QueryLiquidationPriceResp\x12>\n\x11liquidation_price\x18\x01 \x01(\tB#\xc8\xde\x1f\x00\xda\xde\x1f\x1b\x63osmossdk.io/math.LegacyDec\"\x1a\n\x18QueryPremiumIndexConfReq\"Z\n\x19QueryPremiumIndexConfResp\x12=\n\x12premium_index_conf\x18\x01 \x01(\x0b\x32\x1b.fx.dex.v1.PremiumIndexConfB\x04\xc8\xde\x1f\x00\"!\n\x1fQueryNeedToLiquidationPosIdsReq\"g\n\x1fQueryNeedToLiquidatorPosIdsResp\x12\x34\n\x0cposition_ids\x18\x01 \x03(\tB\x1e\xc8\xde\x1f\x00\xda\xde\x1f\x16\x63osmossdk.io/math.Uint\x12\x0e\n\x06height\x18\x02 \x01(\x03\"\x17\n\x15QueryAccountNumberReq\"\'\n\x16QueryAccountNumberResp\x12\r\n\x05value\x18\x01 \x01(\x04\x32\xb2\x12\n\x05Query\x12Y\n\x0bQueryParams\x12\x19.fx.dex.v1.QueryParamsReq\x1a\x1a.fx.dex.v1.QueryParamsResp\"\x13\x82\xd3\xe4\x93\x02\r\x12\x0b/dex/params\x12\x61\n\x0bQueryOrders\x12\x1d.fx.dex.v1.QueryOrdersRequest\x1a\x1e.fx.dex.v1.QueryOrdersResponse\"\x13\x82\xd3\xe4\x93\x02\r\x12\x0b/dex/orders\x12]\n\nQueryOrder\x12\x1c.fx.dex.v1.QueryOrderRequest\x1a\x1d.fx.dex.v1.QueryOrderResponse\"\x12\x82\xd3\xe4\x93\x02\x0c\x12\n/dex/order\x12\x65\n\x0eQueryOrderbook\x12\x1c.fx.dex.v1.QueryOrderbookReq\x1a\x1d.fx.dex.v1.QueryOrderbookResp\"\x16\x82\xd3\xe4\x93\x02\x10\x12\x0e/dex/orderbook\x12\x65\n\x0eQueryPositions\x12\x1c.fx.dex.v1.QueryPositionsReq\x1a\x1d.fx.dex.v1.QueryPositionsResp\"\x16\x82\xd3\xe4\x93\x02\x10\x12\x0e/dex/positions\x12\x61\n\rQueryPosition\x12\x1b.fx.dex.v1.QueryPositionReq\x1a\x1c.fx.dex.v1.QueryPositionResp\"\x15\x82\xd3\xe4\x93\x02\x0f\x12\r/dex/position\x12]\n\x0cQueryFunding\x12\x1a.fx.dex.v1.QueryFundingReq\x1a\x1b.fx.dex.v1.QueryFundingResp\"\x14\x82\xd3\xe4\x93\x02\x0e\x12\x0c/dex/funding\x12~\n\x15QueryPairFundingRates\x12#.fx.dex.v1.QueryPairFundingRatesReq\x1a$.fx.dex.v1.QueryPairFundingRatesResp\"\x1a\x82\xd3\xe4\x93\x02\x14\x12\x12/dex/funding_rates\x12n\n\x10QueryFundingTime\x12\x1e.fx.dex.v1.QueryFundingTimeReq\x1a\x1f.fx.dex.v1.QueryFundingTimeResp\"\x19\x82\xd3\xe4\x93\x02\x13\x12\x11/dex/funding_time\x12\x66\n\x0eQueryDealPrice\x12\x1c.fx.dex.v1.QueryDealPriceReq\x1a\x1d.fx.dex.v1.QueryDealPriceResp\"\x17\x82\xd3\xe4\x93\x02\x11\x12\x0f/dex/deal_price\x12n\n\x10QueryMatchResult\x12\x1e.fx.dex.v1.QueryMatchResultReq\x1a\x1f.fx.dex.v1.QueryMatchResultResp\"\x19\x82\xd3\xe4\x93\x02\x13\x12\x11/dex/match_result\x12\x66\n\x0eQueryMarkPrice\x12\x1c.fx.dex.v1.QueryMarkPriceReq\x1a\x1d.fx.dex.v1.QueryMarkPriceResp\"\x17\x82\xd3\xe4\x93\x02\x11\x12\x0f/dex/mark_price\x12\x88\x01\n\x17QueryMarkAndOraclePrice\x12%.fx.dex.v1.QueryMarkAndOraclePriceReq\x1a&.fx.dex.v1.QueryMarkAndOraclePriceResp\"\x1e\x82\xd3\xe4\x93\x02\x18\x12\x16/dex/mark_oracle_price\x12\x82\x01\n\x15QueryLiquidationPrice\x12#.fx.dex.v1.QueryLiquidationPriceReq\x1a$.fx.dex.v1.QueryLiquidationPriceResp\"\x1e\x82\xd3\xe4\x93\x02\x18\x12\x16/dex/liquidation_price\x12`\n\x12QueryMovingAverage\x12\x15.fx.dex.v1.QueryMAReq\x1a\x16.fx.dex.v1.QueryMAResp\"\x1b\x82\xd3\xe4\x93\x02\x15\x12\x13/dex/moving_average\x12z\n\x13QueryStoreStatistic\x12!.fx.dex.v1.QueryStoreStatisticReq\x1a\".fx.dex.v1.QueryStoreStatisticResp\"\x1c\x82\xd3\xe4\x93\x02\x16\x12\x14/dex/store_statistic\x12\x66\n\x0eQueryIsFunding\x12\x1c.fx.dex.v1.QueryIsFundingReq\x1a\x1d.fx.dex.v1.QueryIsFundingResp\"\x17\x82\xd3\xe4\x93\x02\x11\x12\x0f/dex/is_funding\x12\x83\x01\n\x15QueryPremiumIndexConf\x12#.fx.dex.v1.QueryPremiumIndexConfReq\x1a$.fx.dex.v1.QueryPremiumIndexConfResp\"\x1f\x82\xd3\xe4\x93\x02\x19\x12\x17/dex/premium_index_conf\x12\x98\x01\n\x1cQueryNeedToLiquidationPosIds\x12*.fx.dex.v1.QueryNeedToLiquidationPosIdsReq\x1a*.fx.dex.v1.QueryNeedToLiquidatorPosIdsResp\" \x82\xd3\xe4\x93\x02\x1a\x12\x18/dex/liquidation_pos_ids\x12v\n\x12QueryAccountNumber\x12 .fx.dex.v1.QueryAccountNumberReq\x1a!.fx.dex.v1.QueryAccountNumberResp\"\x1b\x82\xd3\xe4\x93\x02\x15\x12\x13/dex/account_number\x12]\n\x0cQueryReserve\x12\x1a.fx.dex.v1.QueryReserveReq\x1a\x1b.fx.dex.v1.QueryReserveResp\"\x14\x82\xd3\xe4\x93\x02\x0e\x12\x0c/dex/reserveB-Z+github.com/marginxio/marginx/x/dex/v1/typesb\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'fx.dex.v1.query_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:
  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'Z+github.com/marginxio/marginx/x/dex/v1/types'
  _QUERYMARESP.fields_by_name['average_price']._options = None
  _QUERYMARESP.fields_by_name['average_price']._serialized_options = b'\310\336\037\000\332\336\037\033cosmossdk.io/math.LegacyDec'
  _QUERYMARESP.fields_by_name['price2']._options = None
  _QUERYMARESP.fields_by_name['price2']._serialized_options = b'\310\336\037\000\332\336\037\033cosmossdk.io/math.LegacyDec'
  _QUERYMARKANDORACLEPRICERESP.fields_by_name['prices']._options = None
  _QUERYMARKANDORACLEPRICERESP.fields_by_name['prices']._serialized_options = b'\310\336\037\000'
  _QUERYPARAMSRESP.fields_by_name['params']._options = None
  _QUERYPARAMSRESP.fields_by_name['params']._serialized_options = b'\310\336\037\000'
  _QUERYORDERSREQUEST.fields_by_name['page']._options = None
  _QUERYORDERSREQUEST.fields_by_name['page']._serialized_options = b'\310\336\037\000\332\336\037\026cosmossdk.io/math.Uint'
  _QUERYORDERSREQUEST.fields_by_name['limit']._options = None
  _QUERYORDERSREQUEST.fields_by_name['limit']._serialized_options = b'\310\336\037\000\332\336\037\026cosmossdk.io/math.Uint'
  _QUERYORDERSRESPONSE.fields_by_name['orders']._options = None
  _QUERYORDERSRESPONSE.fields_by_name['orders']._serialized_options = b'\310\336\037\000'
  _QUERYORDERRESPONSE.fields_by_name['order']._options = None
  _QUERYORDERRESPONSE.fields_by_name['order']._serialized_options = b'\310\336\037\000'
  _QUERYORDERBOOKRESP.fields_by_name['Asks']._options = None
  _QUERYORDERBOOKRESP.fields_by_name['Asks']._serialized_options = b'\310\336\037\000'
  _QUERYORDERBOOKRESP.fields_by_name['Bids']._options = None
  _QUERYORDERBOOKRESP.fields_by_name['Bids']._serialized_options = b'\310\336\037\000'
  _QUERYPOSITIONSREQ.fields_by_name['page']._options = None
  _QUERYPOSITIONSREQ.fields_by_name['page']._serialized_options = b'\310\336\037\000\332\336\037\026cosmossdk.io/math.Uint'
  _QUERYPOSITIONSREQ.fields_by_name['limit']._options = None
  _QUERYPOSITIONSREQ.fields_by_name['limit']._serialized_options = b'\310\336\037\000\332\336\037\026cosmossdk.io/math.Uint'
  _QUERYPOSITIONSRESP.fields_by_name['positions']._options = None
  _QUERYPOSITIONSRESP.fields_by_name['positions']._serialized_options = b'\310\336\037\000'
  _QUERYPOSITIONRESP.fields_by_name['positions']._options = None
  _QUERYPOSITIONRESP.fields_by_name['positions']._serialized_options = b'\310\336\037\000'
  _QUERYFUNDINGRESP.fields_by_name['funding']._options = None
  _QUERYFUNDINGRESP.fields_by_name['funding']._serialized_options = b'\310\336\037\000'
  _QUERYPAIRFUNDINGRATESRESP.fields_by_name['pair_funding_rates']._options = None
  _QUERYPAIRFUNDINGRATESRESP.fields_by_name['pair_funding_rates']._serialized_options = b'\310\336\037\000'
  _QUERYFUNDINGTIMERESP.fields_by_name['funding_time']._options = None
  _QUERYFUNDINGTIMERESP.fields_by_name['funding_time']._serialized_options = b'\310\336\037\000'
  _PAIRPRICE.fields_by_name['price']._options = None
  _PAIRPRICE.fields_by_name['price']._serialized_options = b'\310\336\037\000\332\336\037\033cosmossdk.io/math.LegacyDec'
  _QUERYDEALPRICERESP.fields_by_name['pair_price']._options = None
  _QUERYDEALPRICERESP.fields_by_name['pair_price']._serialized_options = b'\310\336\037\000'
  _QUERYMATCHRESULTRESP.fields_by_name['match_result']._options = None
  _QUERYMATCHRESULTRESP.fields_by_name['match_result']._serialized_options = b'\310\336\037\000'
  _QUERYMARKPRICERESP.fields_by_name['pair_mark_price']._options = None
  _QUERYMARKPRICERESP.fields_by_name['pair_mark_price']._serialized_options = b'\310\336\037\000'
  _MARKANDORACLEPRICE.fields_by_name['mark_price']._options = None
  _MARKANDORACLEPRICE.fields_by_name['mark_price']._serialized_options = b'\310\336\037\000\332\336\037\033cosmossdk.io/math.LegacyDec'
  _MARKANDORACLEPRICE.fields_by_name['oracle_price']._options = None
  _MARKANDORACLEPRICE.fields_by_name['oracle_price']._serialized_options = b'\310\336\037\000\332\336\037\033cosmossdk.io/math.LegacyDec'
  _QUERYLIQUIDATIONPRICEREQ.fields_by_name['entry_price']._options = None
  _QUERYLIQUIDATIONPRICEREQ.fields_by_name['entry_price']._serialized_options = b'\310\336\037\000\332\336\037\033cosmossdk.io/math.LegacyDec'
  _QUERYLIQUIDATIONPRICEREQ.fields_by_name['quantity']._options = None
  _QUERYLIQUIDATIONPRICEREQ.fields_by_name['quantity']._serialized_options = b'\310\336\037\000\332\336\037\033cosmossdk.io/math.LegacyDec'
  _QUERYLIQUIDATIONPRICEREQ.fields_by_name['wallet_balance']._options = None
  _QUERYLIQUIDATIONPRICEREQ.fields_by_name['wallet_balance']._serialized_options = b'\310\336\037\000\332\336\037\033cosmossdk.io/math.LegacyDec'
  _QUERYLIQUIDATIONPRICERESP.fields_by_name['liquidation_price']._options = None
  _QUERYLIQUIDATIONPRICERESP.fields_by_name['liquidation_price']._serialized_options = b'\310\336\037\000\332\336\037\033cosmossdk.io/math.LegacyDec'
  _QUERYPREMIUMINDEXCONFRESP.fields_by_name['premium_index_conf']._options = None
  _QUERYPREMIUMINDEXCONFRESP.fields_by_name['premium_index_conf']._serialized_options = b'\310\336\037\000'
  _QUERYNEEDTOLIQUIDATORPOSIDSRESP.fields_by_name['position_ids']._options = None
  _QUERYNEEDTOLIQUIDATORPOSIDSRESP.fields_by_name['position_ids']._serialized_options = b'\310\336\037\000\332\336\037\026cosmossdk.io/math.Uint'
  _QUERY.methods_by_name['QueryParams']._options = None
  _QUERY.methods_by_name['QueryParams']._serialized_options = b'\202\323\344\223\002\r\022\013/dex/params'
  _QUERY.methods_by_name['QueryOrders']._options = None
  _QUERY.methods_by_name['QueryOrders']._serialized_options = b'\202\323\344\223\002\r\022\013/dex/orders'
  _QUERY.methods_by_name['QueryOrder']._options = None
  _QUERY.methods_by_name['QueryOrder']._serialized_options = b'\202\323\344\223\002\014\022\n/dex/order'
  _QUERY.methods_by_name['QueryOrderbook']._options = None
  _QUERY.methods_by_name['QueryOrderbook']._serialized_options = b'\202\323\344\223\002\020\022\016/dex/orderbook'
  _QUERY.methods_by_name['QueryPositions']._options = None
  _QUERY.methods_by_name['QueryPositions']._serialized_options = b'\202\323\344\223\002\020\022\016/dex/positions'
  _QUERY.methods_by_name['QueryPosition']._options = None
  _QUERY.methods_by_name['QueryPosition']._serialized_options = b'\202\323\344\223\002\017\022\r/dex/position'
  _QUERY.methods_by_name['QueryFunding']._options = None
  _QUERY.methods_by_name['QueryFunding']._serialized_options = b'\202\323\344\223\002\016\022\014/dex/funding'
  _QUERY.methods_by_name['QueryPairFundingRates']._options = None
  _QUERY.methods_by_name['QueryPairFundingRates']._serialized_options = b'\202\323\344\223\002\024\022\022/dex/funding_rates'
  _QUERY.methods_by_name['QueryFundingTime']._options = None
  _QUERY.methods_by_name['QueryFundingTime']._serialized_options = b'\202\323\344\223\002\023\022\021/dex/funding_time'
  _QUERY.methods_by_name['QueryDealPrice']._options = None
  _QUERY.methods_by_name['QueryDealPrice']._serialized_options = b'\202\323\344\223\002\021\022\017/dex/deal_price'
  _QUERY.methods_by_name['QueryMatchResult']._options = None
  _QUERY.methods_by_name['QueryMatchResult']._serialized_options = b'\202\323\344\223\002\023\022\021/dex/match_result'
  _QUERY.methods_by_name['QueryMarkPrice']._options = None
  _QUERY.methods_by_name['QueryMarkPrice']._serialized_options = b'\202\323\344\223\002\021\022\017/dex/mark_price'
  _QUERY.methods_by_name['QueryMarkAndOraclePrice']._options = None
  _QUERY.methods_by_name['QueryMarkAndOraclePrice']._serialized_options = b'\202\323\344\223\002\030\022\026/dex/mark_oracle_price'
  _QUERY.methods_by_name['QueryLiquidationPrice']._options = None
  _QUERY.methods_by_name['QueryLiquidationPrice']._serialized_options = b'\202\323\344\223\002\030\022\026/dex/liquidation_price'
  _QUERY.methods_by_name['QueryMovingAverage']._options = None
  _QUERY.methods_by_name['QueryMovingAverage']._serialized_options = b'\202\323\344\223\002\025\022\023/dex/moving_average'
  _QUERY.methods_by_name['QueryStoreStatistic']._options = None
  _QUERY.methods_by_name['QueryStoreStatistic']._serialized_options = b'\202\323\344\223\002\026\022\024/dex/store_statistic'
  _QUERY.methods_by_name['QueryIsFunding']._options = None
  _QUERY.methods_by_name['QueryIsFunding']._serialized_options = b'\202\323\344\223\002\021\022\017/dex/is_funding'
  _QUERY.methods_by_name['QueryPremiumIndexConf']._options = None
  _QUERY.methods_by_name['QueryPremiumIndexConf']._serialized_options = b'\202\323\344\223\002\031\022\027/dex/premium_index_conf'
  _QUERY.methods_by_name['QueryNeedToLiquidationPosIds']._options = None
  _QUERY.methods_by_name['QueryNeedToLiquidationPosIds']._serialized_options = b'\202\323\344\223\002\032\022\030/dex/liquidation_pos_ids'
  _QUERY.methods_by_name['QueryAccountNumber']._options = None
  _QUERY.methods_by_name['QueryAccountNumber']._serialized_options = b'\202\323\344\223\002\025\022\023/dex/account_number'
  _QUERY.methods_by_name['QueryReserve']._options = None
  _QUERY.methods_by_name['QueryReserve']._serialized_options = b'\202\323\344\223\002\016\022\014/dex/reserve'
  _globals['_QUERYRESERVEREQ']._serialized_start=209
  _globals['_QUERYRESERVEREQ']._serialized_end=226
  _globals['_QUERYRESERVERESP']._serialized_start=228
  _globals['_QUERYRESERVERESP']._serialized_end=283
  _globals['_QUERYISFUNDINGREQ']._serialized_start=285
  _globals['_QUERYISFUNDINGREQ']._serialized_end=304
  _globals['_QUERYISFUNDINGRESP']._serialized_start=306
  _globals['_QUERYISFUNDINGRESP']._serialized_end=346
  _globals['_QUERYSTORESTATISTICREQ']._serialized_start=348
  _globals['_QUERYSTORESTATISTICREQ']._serialized_end=372
  _globals['_QUERYSTORESTATISTICRESP']._serialized_start=374
  _globals['_QUERYSTORESTATISTICRESP']._serialized_end=474
  _globals['_QUERYMAREQ']._serialized_start=476
  _globals['_QUERYMAREQ']._serialized_end=505
  _globals['_QUERYMARESP']._serialized_start=507
  _globals['_QUERYMARESP']._serialized_end=633
  _globals['_QUERYMARKANDORACLEPRICEREQ']._serialized_start=635
  _globals['_QUERYMARKANDORACLEPRICEREQ']._serialized_end=699
  _globals['_QUERYMARKANDORACLEPRICERESP']._serialized_start=701
  _globals['_QUERYMARKANDORACLEPRICERESP']._serialized_end=783
  _globals['_QUERYPARAMSREQ']._serialized_start=785
  _globals['_QUERYPARAMSREQ']._serialized_end=801
  _globals['_QUERYPARAMSRESP']._serialized_start=803
  _globals['_QUERYPARAMSRESP']._serialized_end=861
  _globals['_QUERYORDERSREQUEST']._serialized_start=864
  _globals['_QUERYORDERSREQUEST']._serialized_end=1009
  _globals['_QUERYORDERSRESPONSE']._serialized_start=1011
  _globals['_QUERYORDERSRESPONSE']._serialized_end=1072
  _globals['_QUERYORDERREQUEST']._serialized_start=1074
  _globals['_QUERYORDERREQUEST']._serialized_end=1111
  _globals['_QUERYORDERRESPONSE']._serialized_start=1113
  _globals['_QUERYORDERRESPONSE']._serialized_end=1172
  _globals['_QUERYORDERBOOKREQ']._serialized_start=1174
  _globals['_QUERYORDERBOOKREQ']._serialized_end=1210
  _globals['_QUERYORDERBOOKRESP']._serialized_start=1212
  _globals['_QUERYORDERBOOKRESP']._serialized_end=1318
  _globals['_QUERYPOSITIONSREQ']._serialized_start=1321
  _globals['_QUERYPOSITIONSREQ']._serialized_end=1509
  _globals['_QUERYPOSITIONSRESP']._serialized_start=1511
  _globals['_QUERYPOSITIONSRESP']._serialized_end=1577
  _globals['_QUERYPOSITIONREQ']._serialized_start=1579
  _globals['_QUERYPOSITIONREQ']._serialized_end=1629
  _globals['_QUERYPOSITIONRESP']._serialized_start=1631
  _globals['_QUERYPOSITIONRESP']._serialized_end=1696
  _globals['_QUERYFUNDINGREQ']._serialized_start=1698
  _globals['_QUERYFUNDINGREQ']._serialized_end=1715
  _globals['_QUERYFUNDINGRESP']._serialized_start=1717
  _globals['_QUERYFUNDINGRESP']._serialized_end=1778
  _globals['_QUERYPAIRFUNDINGRATESREQ']._serialized_start=1780
  _globals['_QUERYPAIRFUNDINGRATESREQ']._serialized_end=1832
  _globals['_QUERYPAIRFUNDINGRATESRESP']._serialized_start=1834
  _globals['_QUERYPAIRFUNDINGRATESRESP']._serialized_end=1923
  _globals['_QUERYFUNDINGTIMEREQ']._serialized_start=1925
  _globals['_QUERYFUNDINGTIMEREQ']._serialized_end=1946
  _globals['_QUERYFUNDINGTIMERESP']._serialized_start=1948
  _globals['_QUERYFUNDINGTIMERESP']._serialized_end=2022
  _globals['_PAIRPRICE']._serialized_start=2024
  _globals['_PAIRPRICE']._serialized_end=2104
  _globals['_QUERYDEALPRICEREQ']._serialized_start=2106
  _globals['_QUERYDEALPRICEREQ']._serialized_end=2161
  _globals['_QUERYDEALPRICERESP']._serialized_start=2163
  _globals['_QUERYDEALPRICERESP']._serialized_end=2231
  _globals['_QUERYMATCHRESULTREQ']._serialized_start=2233
  _globals['_QUERYMATCHRESULTREQ']._serialized_end=2293
  _globals['_QUERYMATCHRESULTRESP']._serialized_start=2295
  _globals['_QUERYMATCHRESULTRESP']._serialized_end=2369
  _globals['_QUERYMARKPRICEREQ']._serialized_start=2371
  _globals['_QUERYMARKPRICEREQ']._serialized_end=2426
  _globals['_QUERYMARKPRICERESP']._serialized_start=2428
  _globals['_QUERYMARKPRICERESP']._serialized_end=2501
  _globals['_MARKANDORACLEPRICE']._serialized_start=2504
  _globals['_MARKANDORACLEPRICE']._serialized_end=2657
  _globals['_QUERYLIQUIDATIONPRICEREQ']._serialized_start=2660
  _globals['_QUERYLIQUIDATIONPRICEREQ']._serialized_end=2939
  _globals['_QUERYLIQUIDATIONPRICERESP']._serialized_start=2941
  _globals['_QUERYLIQUIDATIONPRICERESP']._serialized_end=3032
  _globals['_QUERYPREMIUMINDEXCONFREQ']._serialized_start=3034
  _globals['_QUERYPREMIUMINDEXCONFREQ']._serialized_end=3060
  _globals['_QUERYPREMIUMINDEXCONFRESP']._serialized_start=3062
  _globals['_QUERYPREMIUMINDEXCONFRESP']._serialized_end=3152
  _globals['_QUERYNEEDTOLIQUIDATIONPOSIDSREQ']._serialized_start=3154
  _globals['_QUERYNEEDTOLIQUIDATIONPOSIDSREQ']._serialized_end=3187
  _globals['_QUERYNEEDTOLIQUIDATORPOSIDSRESP']._serialized_start=3189
  _globals['_QUERYNEEDTOLIQUIDATORPOSIDSRESP']._serialized_end=3292
  _globals['_QUERYACCOUNTNUMBERREQ']._serialized_start=3294
  _globals['_QUERYACCOUNTNUMBERREQ']._serialized_end=3317
  _globals['_QUERYACCOUNTNUMBERRESP']._serialized_start=3319
  _globals['_QUERYACCOUNTNUMBERRESP']._serialized_end=3358
  _globals['_QUERY']._serialized_start=3361
  _globals['_QUERY']._serialized_end=5715
# @@protoc_insertion_point(module_scope)



# Generated by the gRPC Python protocol compiler plugin. DO NOT EDIT!
"""Client and server classes corresponding to protobuf-defined services."""
import grpc

from fxsdk.x.fx.migrate.v1 import query_pb2 as fx_dot_migrate_dot_v1_dot_query__pb2


class QueryStub(object):
    """Query provides defines the gRPC querier service.
    """

    def __init__(self, channel):
        """Constructor.

        Args:
            channel: A grpc.Channel.
        """
        self.MigrateRecord = channel.unary_unary(
                '/fx.migrate.v1.Query/MigrateRecord',
                request_serializer=fx_dot_migrate_dot_v1_dot_query__pb2.QueryMigrateRecordRequest.SerializeToString,
                response_deserializer=fx_dot_migrate_dot_v1_dot_query__pb2.QueryMigrateRecordResponse.FromString,
                )
        self.MigrateCheckAccount = channel.unary_unary(
                '/fx.migrate.v1.Query/MigrateCheckAccount',
                request_serializer=fx_dot_migrate_dot_v1_dot_query__pb2.QueryMigrateCheckAccountRequest.SerializeToString,
                response_deserializer=fx_dot_migrate_dot_v1_dot_query__pb2.QueryMigrateCheckAccountResponse.FromString,
                )


class QueryServicer(object):
    """Query provides defines the gRPC querier service.
    """

    def MigrateRecord(self, request, context):
        """DenomTrace queries a denomination trace information.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def MigrateCheckAccount(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')


def add_QueryServicer_to_server(servicer, server):
    rpc_method_handlers = {
            'MigrateRecord': grpc.unary_unary_rpc_method_handler(
                    servicer.MigrateRecord,
                    request_deserializer=fx_dot_migrate_dot_v1_dot_query__pb2.QueryMigrateRecordRequest.FromString,
                    response_serializer=fx_dot_migrate_dot_v1_dot_query__pb2.QueryMigrateRecordResponse.SerializeToString,
            ),
            'MigrateCheckAccount': grpc.unary_unary_rpc_method_handler(
                    servicer.MigrateCheckAccount,
                    request_deserializer=fx_dot_migrate_dot_v1_dot_query__pb2.QueryMigrateCheckAccountRequest.FromString,
                    response_serializer=fx_dot_migrate_dot_v1_dot_query__pb2.QueryMigrateCheckAccountResponse.SerializeToString,
            ),
    }
    generic_handler = grpc.method_handlers_generic_handler(
            'fx.migrate.v1.Query', rpc_method_handlers)
    server.add_generic_rpc_handlers((generic_handler,))


 # This class is part of an EXPERIMENTAL API.
class Query(object):
    """Query provides defines the gRPC querier service.
    """

    @staticmethod
    def MigrateRecord(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/fx.migrate.v1.Query/MigrateRecord',
            fx_dot_migrate_dot_v1_dot_query__pb2.QueryMigrateRecordRequest.SerializeToString,
            fx_dot_migrate_dot_v1_dot_query__pb2.QueryMigrateRecordResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def MigrateCheckAccount(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/fx.migrate.v1.Query/MigrateCheckAccount',
            fx_dot_migrate_dot_v1_dot_query__pb2.QueryMigrateCheckAccountRequest.SerializeToString,
            fx_dot_migrate_dot_v1_dot_query__pb2.QueryMigrateCheckAccountResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)



import unittest

from fxsdk.client.grpc import Client

grpc_cli = Client('localhost:9090')

test_address = "mx17w0adeg64ky0daxwd2ugyuneellmjgnxqezt98"


class TestGrpcClientQuery(unittest.TestCase):

    def test_query_balances(self):
        balances = grpc_cli.query_all_balances(address=test_address)
        print(balances)

    def test_query_balance(self):
        balance = grpc_cli.query_balance(address=test_address, denom="FX")
        print(balance)

        balances = grpc_cli.query_balance(address=test_address, denom="FX")
        print(balances)

    def test_query_account(self):
        account = grpc_cli.query_account_info(address=test_address)
        print(account)

    def test_query_module_accounts(self):
        module_accounts = grpc_cli.query_module_accounts()
        print(module_accounts)

    def test_query_total_supply(self):
        supply = grpc_cli.query_total_supply()
        print(supply)

    def test_query_validators(self):
        validators = grpc_cli.query_validators()
        print(validators)

    def test_query_gas_prices(self):
        res = grpc_cli.query_gas_prices()
        print(res)

    def test_query_node_info(self):
        res = grpc_cli.query_node_info()
        print(res)

    def test_query_latest_block(self):
        res = grpc_cli.query_latest_block()
        print(res)

    def test_query_block_by_height(self):
        res = grpc_cli.query_block_by_height(height=1)
        print(res)

    def test_avg_block_time_interval(self):
        res = grpc_cli.avg_block_time_interval()
        print(res)


if __name__ == '__main__':
    unittest.main()



import bech32

from eth_utils import to_checksum_address, is_hex_address, to_canonical_address

DEFAULT_BECH32_PREFIX = "fx"


class Address:
    def __init__(self, addr: bytes, prefix: str):
        if prefix is None or prefix == "":
            raise Exception("invalid prefix")
        if len(addr) != 20:
            raise Exception("invalid address")
        self._data = addr
        self._prefix = prefix

    @classmethod
    def from_str(cls, addr: str):
        if is_hex_address(addr):
            data = bytes(to_canonical_address(addr))
            return cls(addr=data, prefix="0x")
        else:
            prefix, data = bech32.bech32_decode(addr)
            data = bytes(bech32.convertbits(data, 5, 8, False))
            return cls(addr=data, prefix=prefix)

    def to_string(self, prefix: str = None) -> str:
        prefix = self._prefix if prefix is None else prefix
        if prefix == "0x":
            return self.to_hex()
        data = bech32.convertbits(self._data, 8, 5)
        return bech32.bech32_encode(prefix, data)

    def to_hex(self) -> str:
        return to_checksum_address(self._data)

    def to_bytes(self) -> bytes:
        return self._data

    def prefix(self) -> str:
        return self._prefix



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: fx/dex/v1/order.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from fxsdk.x.gogoproto import gogo_pb2 as gogoproto_dot_gogo__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x15\x66x/dex/v1/order.proto\x12\tfx.dex.v1\x1a\x14gogoproto/gogo.proto\"\x17\n\x08OrderIDs\x12\x0b\n\x03ids\x18\x01 \x03(\t\"\xa5\x05\n\x05Order\x12\n\n\x02id\x18\x01 \x01(\t\x12@\n\x05owner\x18\x02 \x01(\x0c\x42\x31\xfa\xde\x1f-github.com/cosmos/cosmos-sdk/types.AccAddress\x12\x0f\n\x07pair_id\x18\x03 \x01(\t\x12\'\n\tdirection\x18\x04 \x01(\x0e\x32\x14.fx.dex.v1.Direction\x12\x32\n\x05price\x18\x05 \x01(\tB#\xc8\xde\x1f\x00\xda\xde\x1f\x1b\x63osmossdk.io/math.LegacyDec\x12:\n\rbase_quantity\x18\x06 \x01(\tB#\xc8\xde\x1f\x00\xda\xde\x1f\x1b\x63osmossdk.io/math.LegacyDec\x12;\n\x0equote_quantity\x18\x07 \x01(\tB#\xc8\xde\x1f\x00\xda\xde\x1f\x1b\x63osmossdk.io/math.LegacyDec\x12<\n\x0f\x66illed_quantity\x18\x08 \x01(\tB#\xc8\xde\x1f\x00\xda\xde\x1f\x1b\x63osmossdk.io/math.LegacyDec\x12=\n\x10\x66illed_avg_price\x18\t \x01(\tB#\xc8\xde\x1f\x00\xda\xde\x1f\x1b\x63osmossdk.io/math.LegacyDec\x12\x10\n\x08leverage\x18\n \x01(\x03\x12&\n\x06status\x18\x0b \x01(\x0e\x32\x16.fx.dex.v1.OrderStatus\x12(\n\norder_type\x18\x0c \x01(\x0e\x32\x14.fx.dex.v1.OrderType\x12\x41\n\x08\x63ost_fee\x18\r \x01(\tB/\xc8\xde\x1f\x00\xda\xde\x1f\'github.com/cosmos/cosmos-sdk/types.Coin\x12\x43\n\nlocked_fee\x18\x0e \x01(\tB/\xc8\xde\x1f\x00\xda\xde\x1f\'github.com/cosmos/cosmos-sdk/types.Coin\"0\n\x06Orders\x12&\n\x06orders\x18\x01 \x03(\x0b\x32\x10.fx.dex.v1.OrderB\x04\xc8\xde\x1f\x00*\x8d\x01\n\tDirection\x12\x12\n\x04\x42OTH\x10\x00\x1a\x08\x8a\x9d \x04\x42OTH\x12\x10\n\x03\x42UY\x10\x01\x1a\x07\x8a\x9d \x03\x42UY\x12\x12\n\x04SELL\x10\x02\x1a\x08\x8a\x9d \x04SELL\x12\x1c\n\tMarketBuy\x10\x03\x1a\r\x8a\x9d \tMarketBuy\x12\x1e\n\nMarketSell\x10\x04\x1a\x0e\x8a\x9d \nMarketSell\x1a\x08\x88\xa3\x1e\x00\xa8\xa4\x1e\x01*\xc3\x03\n\x0bOrderStatus\x12$\n\rORDER_PENDING\x10\x00\x1a\x11\x8a\x9d \rORDER_PENDING\x12\"\n\x0cORDER_FILLED\x10\x01\x1a\x10\x8a\x9d \x0cORDER_FILLED\x12\x32\n\x14ORDER_PARTIAL_FILLED\x10\x02\x1a\x18\x8a\x9d \x14ORDER_PARTIAL_FILLED\x12(\n\x0fORDER_CANCELLED\x10\x03\x1a\x13\x8a\x9d \x0fORDER_CANCELLED\x12\x46\n\x1eORDER_PARTIAL_FILLED_CANCELLED\x10\x04\x1a\"\x8a\x9d \x1eORDER_PARTIAL_FILLED_CANCELLED\x12\x42\n\x1cORDER_PARTIAL_FILLED_EXPIRED\x10\x05\x1a \x8a\x9d \x1cORDER_PARTIAL_FILLED_EXPIRED\x12$\n\rORDER_EXPIRED\x10\x06\x1a\x11\x8a\x9d \rORDER_EXPIRED\x12\"\n\x0cORDER_GASOUT\x10\x07\x1a\x10\x8a\x9d \x0cORDER_GASOUT\x12,\n\x11ORDER_PRICE_LIMIT\x10\x08\x1a\x15\x8a\x9d \x11ORDER_PRICE_LIMIT\x1a\x08\x88\xa3\x1e\x00\xa8\xa4\x1e\x01*\xc7\x01\n\tOrderType\x12:\n\x18ORDER_TYPE_OPEN_POSITION\x10\x00\x1a\x1c\x8a\x9d \x18ORDER_TYPE_OPEN_POSITION\x12<\n\x19ORDER_TYPE_CLOSE_POSITION\x10\x01\x1a\x1d\x8a\x9d \x19ORDER_TYPE_CLOSE_POSITION\x12\x36\n\x16ORDER_TYPE_LIQUIDATION\x10\x02\x1a\x1a\x8a\x9d \x16ORDER_TYPE_LIQUIDATION\x1a\x08\x88\xa3\x1e\x00\xa8\xa4\x1e\x01\x42\x31Z+github.com/marginxio/marginx/x/dex/v1/types\xa8\xe2\x1e\x01\x62\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'fx.dex.v1.order_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:
  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'Z+github.com/marginxio/marginx/x/dex/v1/types\250\342\036\001'
  _DIRECTION._options = None
  _DIRECTION._serialized_options = b'\210\243\036\000\250\244\036\001'
  _DIRECTION.values_by_name["BOTH"]._options = None
  _DIRECTION.values_by_name["BOTH"]._serialized_options = b'\212\235 \004BOTH'
  _DIRECTION.values_by_name["BUY"]._options = None
  _DIRECTION.values_by_name["BUY"]._serialized_options = b'\212\235 \003BUY'
  _DIRECTION.values_by_name["SELL"]._options = None
  _DIRECTION.values_by_name["SELL"]._serialized_options = b'\212\235 \004SELL'
  _DIRECTION.values_by_name["MarketBuy"]._options = None
  _DIRECTION.values_by_name["MarketBuy"]._serialized_options = b'\212\235 \tMarketBuy'
  _DIRECTION.values_by_name["MarketSell"]._options = None
  _DIRECTION.values_by_name["MarketSell"]._serialized_options = b'\212\235 \nMarketSell'
  _ORDERSTATUS._options = None
  _ORDERSTATUS._serialized_options = b'\210\243\036\000\250\244\036\001'
  _ORDERSTATUS.values_by_name["ORDER_PENDING"]._options = None
  _ORDERSTATUS.values_by_name["ORDER_PENDING"]._serialized_options = b'\212\235 \rORDER_PENDING'
  _ORDERSTATUS.values_by_name["ORDER_FILLED"]._options = None
  _ORDERSTATUS.values_by_name["ORDER_FILLED"]._serialized_options = b'\212\235 \014ORDER_FILLED'
  _ORDERSTATUS.values_by_name["ORDER_PARTIAL_FILLED"]._options = None
  _ORDERSTATUS.values_by_name["ORDER_PARTIAL_FILLED"]._serialized_options = b'\212\235 \024ORDER_PARTIAL_FILLED'
  _ORDERSTATUS.values_by_name["ORDER_CANCELLED"]._options = None
  _ORDERSTATUS.values_by_name["ORDER_CANCELLED"]._serialized_options = b'\212\235 \017ORDER_CANCELLED'
  _ORDERSTATUS.values_by_name["ORDER_PARTIAL_FILLED_CANCELLED"]._options = None
  _ORDERSTATUS.values_by_name["ORDER_PARTIAL_FILLED_CANCELLED"]._serialized_options = b'\212\235 \036ORDER_PARTIAL_FILLED_CANCELLED'
  _ORDERSTATUS.values_by_name["ORDER_PARTIAL_FILLED_EXPIRED"]._options = None
  _ORDERSTATUS.values_by_name["ORDER_PARTIAL_FILLED_EXPIRED"]._serialized_options = b'\212\235 \034ORDER_PARTIAL_FILLED_EXPIRED'
  _ORDERSTATUS.values_by_name["ORDER_EXPIRED"]._options = None
  _ORDERSTATUS.values_by_name["ORDER_EXPIRED"]._serialized_options = b'\212\235 \rORDER_EXPIRED'
  _ORDERSTATUS.values_by_name["ORDER_GASOUT"]._options = None
  _ORDERSTATUS.values_by_name["ORDER_GASOUT"]._serialized_options = b'\212\235 \014ORDER_GASOUT'
  _ORDERSTATUS.values_by_name["ORDER_PRICE_LIMIT"]._options = None
  _ORDERSTATUS.values_by_name["ORDER_PRICE_LIMIT"]._serialized_options = b'\212\235 \021ORDER_PRICE_LIMIT'
  _ORDERTYPE._options = None
  _ORDERTYPE._serialized_options = b'\210\243\036\000\250\244\036\001'
  _ORDERTYPE.values_by_name["ORDER_TYPE_OPEN_POSITION"]._options = None
  _ORDERTYPE.values_by_name["ORDER_TYPE_OPEN_POSITION"]._serialized_options = b'\212\235 \030ORDER_TYPE_OPEN_POSITION'
  _ORDERTYPE.values_by_name["ORDER_TYPE_CLOSE_POSITION"]._options = None
  _ORDERTYPE.values_by_name["ORDER_TYPE_CLOSE_POSITION"]._serialized_options = b'\212\235 \031ORDER_TYPE_CLOSE_POSITION'
  _ORDERTYPE.values_by_name["ORDER_TYPE_LIQUIDATION"]._options = None
  _ORDERTYPE.values_by_name["ORDER_TYPE_LIQUIDATION"]._serialized_options = b'\212\235 \026ORDER_TYPE_LIQUIDATION'
  _ORDER.fields_by_name['owner']._options = None
  _ORDER.fields_by_name['owner']._serialized_options = b'\372\336\037-github.com/cosmos/cosmos-sdk/types.AccAddress'
  _ORDER.fields_by_name['price']._options = None
  _ORDER.fields_by_name['price']._serialized_options = b'\310\336\037\000\332\336\037\033cosmossdk.io/math.LegacyDec'
  _ORDER.fields_by_name['base_quantity']._options = None
  _ORDER.fields_by_name['base_quantity']._serialized_options = b'\310\336\037\000\332\336\037\033cosmossdk.io/math.LegacyDec'
  _ORDER.fields_by_name['quote_quantity']._options = None
  _ORDER.fields_by_name['quote_quantity']._serialized_options = b'\310\336\037\000\332\336\037\033cosmossdk.io/math.LegacyDec'
  _ORDER.fields_by_name['filled_quantity']._options = None
  _ORDER.fields_by_name['filled_quantity']._serialized_options = b'\310\336\037\000\332\336\037\033cosmossdk.io/math.LegacyDec'
  _ORDER.fields_by_name['filled_avg_price']._options = None
  _ORDER.fields_by_name['filled_avg_price']._serialized_options = b'\310\336\037\000\332\336\037\033cosmossdk.io/math.LegacyDec'
  _ORDER.fields_by_name['cost_fee']._options = None
  _ORDER.fields_by_name['cost_fee']._serialized_options = b'\310\336\037\000\332\336\037\'github.com/cosmos/cosmos-sdk/types.Coin'
  _ORDER.fields_by_name['locked_fee']._options = None
  _ORDER.fields_by_name['locked_fee']._serialized_options = b'\310\336\037\000\332\336\037\'github.com/cosmos/cosmos-sdk/types.Coin'
  _ORDERS.fields_by_name['orders']._options = None
  _ORDERS.fields_by_name['orders']._serialized_options = b'\310\336\037\000'
  _globals['_DIRECTION']._serialized_start=814
  _globals['_DIRECTION']._serialized_end=955
  _globals['_ORDERSTATUS']._serialized_start=958
  _globals['_ORDERSTATUS']._serialized_end=1409
  _globals['_ORDERTYPE']._serialized_start=1412
  _globals['_ORDERTYPE']._serialized_end=1611
  _globals['_ORDERIDS']._serialized_start=58
  _globals['_ORDERIDS']._serialized_end=81
  _globals['_ORDER']._serialized_start=84
  _globals['_ORDER']._serialized_end=761
  _globals['_ORDERS']._serialized_start=763
  _globals['_ORDERS']._serialized_end=811
# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: cosmos/group/v1/tx.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from fxsdk.x.gogoproto import gogo_pb2 as gogoproto_dot_gogo__pb2
from fxsdk.x.cosmos_proto import cosmos_pb2 as cosmos__proto_dot_cosmos__pb2
from google.protobuf import any_pb2 as google_dot_protobuf_dot_any__pb2
from fxsdk.x.cosmos.group.v1 import types_pb2 as cosmos_dot_group_dot_v1_dot_types__pb2
from fxsdk.x.cosmos.msg.v1 import msg_pb2 as cosmos_dot_msg_dot_v1_dot_msg__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x18\x63osmos/group/v1/tx.proto\x12\x0f\x63osmos.group.v1\x1a\x14gogoproto/gogo.proto\x1a\x19\x63osmos_proto/cosmos.proto\x1a\x19google/protobuf/any.proto\x1a\x1b\x63osmos/group/v1/types.proto\x1a\x17\x63osmos/msg/v1/msg.proto\"\x8e\x01\n\x0eMsgCreateGroup\x12\'\n\x05\x61\x64min\x18\x01 \x01(\tB\x18\xd2\xb4-\x14\x63osmos.AddressString\x12\x35\n\x07members\x18\x02 \x03(\x0b\x32\x1e.cosmos.group.v1.MemberRequestB\x04\xc8\xde\x1f\x00\x12\x10\n\x08metadata\x18\x03 \x01(\t:\n\x82\xe7\xb0*\x05\x61\x64min\"*\n\x16MsgCreateGroupResponse\x12\x10\n\x08group_id\x18\x01 \x01(\x04\"\x9c\x01\n\x15MsgUpdateGroupMembers\x12\'\n\x05\x61\x64min\x18\x01 \x01(\tB\x18\xd2\xb4-\x14\x63osmos.AddressString\x12\x10\n\x08group_id\x18\x02 \x01(\x04\x12<\n\x0emember_updates\x18\x03 \x03(\x0b\x32\x1e.cosmos.group.v1.MemberRequestB\x04\xc8\xde\x1f\x00:\n\x82\xe7\xb0*\x05\x61\x64min\"\x1f\n\x1dMsgUpdateGroupMembersResponse\"\x89\x01\n\x13MsgUpdateGroupAdmin\x12\'\n\x05\x61\x64min\x18\x01 \x01(\tB\x18\xd2\xb4-\x14\x63osmos.AddressString\x12\x10\n\x08group_id\x18\x02 \x01(\x04\x12+\n\tnew_admin\x18\x03 \x01(\tB\x18\xd2\xb4-\x14\x63osmos.AddressString:\n\x82\xe7\xb0*\x05\x61\x64min\"\x1d\n\x1bMsgUpdateGroupAdminResponse\"q\n\x16MsgUpdateGroupMetadata\x12\'\n\x05\x61\x64min\x18\x01 \x01(\tB\x18\xd2\xb4-\x14\x63osmos.AddressString\x12\x10\n\x08group_id\x18\x02 \x01(\x04\x12\x10\n\x08metadata\x18\x03 \x01(\t:\n\x82\xe7\xb0*\x05\x61\x64min\" \n\x1eMsgUpdateGroupMetadataResponse\"\xc6\x01\n\x14MsgCreateGroupPolicy\x12\'\n\x05\x61\x64min\x18\x01 \x01(\tB\x18\xd2\xb4-\x14\x63osmos.AddressString\x12\x10\n\x08group_id\x18\x02 \x01(\x04\x12\x10\n\x08metadata\x18\x03 \x01(\t\x12Q\n\x0f\x64\x65\x63ision_policy\x18\x04 \x01(\x0b\x32\x14.google.protobuf.AnyB\"\xca\xb4-\x1e\x63osmos.group.v1.DecisionPolicy:\x0e\x88\xa0\x1f\x00\x82\xe7\xb0*\x05\x61\x64min\"I\n\x1cMsgCreateGroupPolicyResponse\x12)\n\x07\x61\x64\x64ress\x18\x01 \x01(\tB\x18\xd2\xb4-\x14\x63osmos.AddressString\"\xb5\x01\n\x19MsgUpdateGroupPolicyAdmin\x12\'\n\x05\x61\x64min\x18\x01 \x01(\tB\x18\xd2\xb4-\x14\x63osmos.AddressString\x12\x36\n\x14group_policy_address\x18\x02 \x01(\tB\x18\xd2\xb4-\x14\x63osmos.AddressString\x12+\n\tnew_admin\x18\x03 \x01(\tB\x18\xd2\xb4-\x14\x63osmos.AddressString:\n\x82\xe7\xb0*\x05\x61\x64min\"\xb3\x02\n\x18MsgCreateGroupWithPolicy\x12\'\n\x05\x61\x64min\x18\x01 \x01(\tB\x18\xd2\xb4-\x14\x63osmos.AddressString\x12\x35\n\x07members\x18\x02 \x03(\x0b\x32\x1e.cosmos.group.v1.MemberRequestB\x04\xc8\xde\x1f\x00\x12\x16\n\x0egroup_metadata\x18\x03 \x01(\t\x12\x1d\n\x15group_policy_metadata\x18\x04 \x01(\t\x12\x1d\n\x15group_policy_as_admin\x18\x05 \x01(\x08\x12Q\n\x0f\x64\x65\x63ision_policy\x18\x06 \x01(\x0b\x32\x14.google.protobuf.AnyB\"\xca\xb4-\x1e\x63osmos.group.v1.DecisionPolicy:\x0e\x88\xa0\x1f\x00\x82\xe7\xb0*\x05\x61\x64min\"l\n MsgCreateGroupWithPolicyResponse\x12\x10\n\x08group_id\x18\x01 \x01(\x04\x12\x36\n\x14group_policy_address\x18\x02 \x01(\tB\x18\xd2\xb4-\x14\x63osmos.AddressString\"#\n!MsgUpdateGroupPolicyAdminResponse\"\xe8\x01\n\"MsgUpdateGroupPolicyDecisionPolicy\x12\'\n\x05\x61\x64min\x18\x01 \x01(\tB\x18\xd2\xb4-\x14\x63osmos.AddressString\x12\x36\n\x14group_policy_address\x18\x02 \x01(\tB\x18\xd2\xb4-\x14\x63osmos.AddressString\x12Q\n\x0f\x64\x65\x63ision_policy\x18\x03 \x01(\x0b\x32\x14.google.protobuf.AnyB\"\xca\xb4-\x1e\x63osmos.group.v1.DecisionPolicy:\x0e\x88\xa0\x1f\x00\x82\xe7\xb0*\x05\x61\x64min\",\n*MsgUpdateGroupPolicyDecisionPolicyResponse\"\x9d\x01\n\x1cMsgUpdateGroupPolicyMetadata\x12\'\n\x05\x61\x64min\x18\x01 \x01(\tB\x18\xd2\xb4-\x14\x63osmos.AddressString\x12\x36\n\x14group_policy_address\x18\x02 \x01(\tB\x18\xd2\xb4-\x14\x63osmos.AddressString\x12\x10\n\x08metadata\x18\x03 \x01(\t:\n\x82\xe7\xb0*\x05\x61\x64min\"&\n$MsgUpdateGroupPolicyMetadataResponse\"\xd1\x01\n\x11MsgSubmitProposal\x12\x36\n\x14group_policy_address\x18\x01 \x01(\tB\x18\xd2\xb4-\x14\x63osmos.AddressString\x12\x11\n\tproposers\x18\x02 \x03(\t\x12\x10\n\x08metadata\x18\x03 \x01(\t\x12&\n\x08messages\x18\x04 \x03(\x0b\x32\x14.google.protobuf.Any\x12#\n\x04\x65xec\x18\x05 \x01(\x0e\x32\x15.cosmos.group.v1.Exec:\x12\x88\xa0\x1f\x00\x82\xe7\xb0*\tproposers\"0\n\x19MsgSubmitProposalResponse\x12\x13\n\x0bproposal_id\x18\x01 \x01(\x04\"c\n\x13MsgWithdrawProposal\x12\x13\n\x0bproposal_id\x18\x01 \x01(\x04\x12)\n\x07\x61\x64\x64ress\x18\x02 \x01(\tB\x18\xd2\xb4-\x14\x63osmos.AddressString:\x0c\x82\xe7\xb0*\x07\x61\x64\x64ress\"\x1d\n\x1bMsgWithdrawProposalResponse\"\xb7\x01\n\x07MsgVote\x12\x13\n\x0bproposal_id\x18\x01 \x01(\x04\x12\'\n\x05voter\x18\x02 \x01(\tB\x18\xd2\xb4-\x14\x63osmos.AddressString\x12+\n\x06option\x18\x03 \x01(\x0e\x32\x1b.cosmos.group.v1.VoteOption\x12\x10\n\x08metadata\x18\x04 \x01(\t\x12#\n\x04\x65xec\x18\x05 \x01(\x0e\x32\x15.cosmos.group.v1.Exec:\n\x82\xe7\xb0*\x05voter\"\x11\n\x0fMsgVoteResponse\"W\n\x07MsgExec\x12\x13\n\x0bproposal_id\x18\x01 \x01(\x04\x12*\n\x08\x65xecutor\x18\x02 \x01(\tB\x18\xd2\xb4-\x14\x63osmos.AddressString:\x0b\x82\xe7\xb0*\x06signer\"J\n\x0fMsgExecResponse\x12\x37\n\x06result\x18\x02 \x01(\x0e\x32\'.cosmos.group.v1.ProposalExecutorResult\"Z\n\rMsgLeaveGroup\x12)\n\x07\x61\x64\x64ress\x18\x01 \x01(\tB\x18\xd2\xb4-\x14\x63osmos.AddressString\x12\x10\n\x08group_id\x18\x02 \x01(\x04:\x0c\x82\xe7\xb0*\x07\x61\x64\x64ress\"\x17\n\x15MsgLeaveGroupResponse**\n\x04\x45xec\x12\x14\n\x10\x45XEC_UNSPECIFIED\x10\x00\x12\x0c\n\x08\x45XEC_TRY\x10\x01\x32\xc3\x0b\n\x03Msg\x12W\n\x0b\x43reateGroup\x12\x1f.cosmos.group.v1.MsgCreateGroup\x1a\'.cosmos.group.v1.MsgCreateGroupResponse\x12l\n\x12UpdateGroupMembers\x12&.cosmos.group.v1.MsgUpdateGroupMembers\x1a..cosmos.group.v1.MsgUpdateGroupMembersResponse\x12\x66\n\x10UpdateGroupAdmin\x12$.cosmos.group.v1.MsgUpdateGroupAdmin\x1a,.cosmos.group.v1.MsgUpdateGroupAdminResponse\x12o\n\x13UpdateGroupMetadata\x12\'.cosmos.group.v1.MsgUpdateGroupMetadata\x1a/.cosmos.group.v1.MsgUpdateGroupMetadataResponse\x12i\n\x11\x43reateGroupPolicy\x12%.cosmos.group.v1.MsgCreateGroupPolicy\x1a-.cosmos.group.v1.MsgCreateGroupPolicyResponse\x12u\n\x15\x43reateGroupWithPolicy\x12).cosmos.group.v1.MsgCreateGroupWithPolicy\x1a\x31.cosmos.group.v1.MsgCreateGroupWithPolicyResponse\x12x\n\x16UpdateGroupPolicyAdmin\x12*.cosmos.group.v1.MsgUpdateGroupPolicyAdmin\x1a\x32.cosmos.group.v1.MsgUpdateGroupPolicyAdminResponse\x12\x93\x01\n\x1fUpdateGroupPolicyDecisionPolicy\x12\x33.cosmos.group.v1.MsgUpdateGroupPolicyDecisionPolicy\x1a;.cosmos.group.v1.MsgUpdateGroupPolicyDecisionPolicyResponse\x12\x81\x01\n\x19UpdateGroupPolicyMetadata\x12-.cosmos.group.v1.MsgUpdateGroupPolicyMetadata\x1a\x35.cosmos.group.v1.MsgUpdateGroupPolicyMetadataResponse\x12`\n\x0eSubmitProposal\x12\".cosmos.group.v1.MsgSubmitProposal\x1a*.cosmos.group.v1.MsgSubmitProposalResponse\x12\x66\n\x10WithdrawProposal\x12$.cosmos.group.v1.MsgWithdrawProposal\x1a,.cosmos.group.v1.MsgWithdrawProposalResponse\x12\x42\n\x04Vote\x12\x18.cosmos.group.v1.MsgVote\x1a .cosmos.group.v1.MsgVoteResponse\x12\x42\n\x04\x45xec\x12\x18.cosmos.group.v1.MsgExec\x1a .cosmos.group.v1.MsgExecResponse\x12T\n\nLeaveGroup\x12\x1e.cosmos.group.v1.MsgLeaveGroup\x1a&.cosmos.group.v1.MsgLeaveGroupResponseB&Z$github.com/cosmos/cosmos-sdk/x/groupb\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'cosmos.group.v1.tx_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:
  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'Z$github.com/cosmos/cosmos-sdk/x/group'
  _MSGCREATEGROUP.fields_by_name['admin']._options = None
  _MSGCREATEGROUP.fields_by_name['admin']._serialized_options = b'\322\264-\024cosmos.AddressString'
  _MSGCREATEGROUP.fields_by_name['members']._options = None
  _MSGCREATEGROUP.fields_by_name['members']._serialized_options = b'\310\336\037\000'
  _MSGCREATEGROUP._options = None
  _MSGCREATEGROUP._serialized_options = b'\202\347\260*\005admin'
  _MSGUPDATEGROUPMEMBERS.fields_by_name['admin']._options = None
  _MSGUPDATEGROUPMEMBERS.fields_by_name['admin']._serialized_options = b'\322\264-\024cosmos.AddressString'
  _MSGUPDATEGROUPMEMBERS.fields_by_name['member_updates']._options = None
  _MSGUPDATEGROUPMEMBERS.fields_by_name['member_updates']._serialized_options = b'\310\336\037\000'
  _MSGUPDATEGROUPMEMBERS._options = None
  _MSGUPDATEGROUPMEMBERS._serialized_options = b'\202\347\260*\005admin'
  _MSGUPDATEGROUPADMIN.fields_by_name['admin']._options = None
  _MSGUPDATEGROUPADMIN.fields_by_name['admin']._serialized_options = b'\322\264-\024cosmos.AddressString'
  _MSGUPDATEGROUPADMIN.fields_by_name['new_admin']._options = None
  _MSGUPDATEGROUPADMIN.fields_by_name['new_admin']._serialized_options = b'\322\264-\024cosmos.AddressString'
  _MSGUPDATEGROUPADMIN._options = None
  _MSGUPDATEGROUPADMIN._serialized_options = b'\202\347\260*\005admin'
  _MSGUPDATEGROUPMETADATA.fields_by_name['admin']._options = None
  _MSGUPDATEGROUPMETADATA.fields_by_name['admin']._serialized_options = b'\322\264-\024cosmos.AddressString'
  _MSGUPDATEGROUPMETADATA._options = None
  _MSGUPDATEGROUPMETADATA._serialized_options = b'\202\347\260*\005admin'
  _MSGCREATEGROUPPOLICY.fields_by_name['admin']._options = None
  _MSGCREATEGROUPPOLICY.fields_by_name['admin']._serialized_options = b'\322\264-\024cosmos.AddressString'
  _MSGCREATEGROUPPOLICY.fields_by_name['decision_policy']._options = None
  _MSGCREATEGROUPPOLICY.fields_by_name['decision_policy']._serialized_options = b'\312\264-\036cosmos.group.v1.DecisionPolicy'
  _MSGCREATEGROUPPOLICY._options = None
  _MSGCREATEGROUPPOLICY._serialized_options = b'\210\240\037\000\202\347\260*\005admin'
  _MSGCREATEGROUPPOLICYRESPONSE.fields_by_name['address']._options = None
  _MSGCREATEGROUPPOLICYRESPONSE.fields_by_name['address']._serialized_options = b'\322\264-\024cosmos.AddressString'
  _MSGUPDATEGROUPPOLICYADMIN.fields_by_name['admin']._options = None
  _MSGUPDATEGROUPPOLICYADMIN.fields_by_name['admin']._serialized_options = b'\322\264-\024cosmos.AddressString'
  _MSGUPDATEGROUPPOLICYADMIN.fields_by_name['group_policy_address']._options = None
  _MSGUPDATEGROUPPOLICYADMIN.fields_by_name['group_policy_address']._serialized_options = b'\322\264-\024cosmos.AddressString'
  _MSGUPDATEGROUPPOLICYADMIN.fields_by_name['new_admin']._options = None
  _MSGUPDATEGROUPPOLICYADMIN.fields_by_name['new_admin']._serialized_options = b'\322\264-\024cosmos.AddressString'
  _MSGUPDATEGROUPPOLICYADMIN._options = None
  _MSGUPDATEGROUPPOLICYADMIN._serialized_options = b'\202\347\260*\005admin'
  _MSGCREATEGROUPWITHPOLICY.fields_by_name['admin']._options = None
  _MSGCREATEGROUPWITHPOLICY.fields_by_name['admin']._serialized_options = b'\322\264-\024cosmos.AddressString'
  _MSGCREATEGROUPWITHPOLICY.fields_by_name['members']._options = None
  _MSGCREATEGROUPWITHPOLICY.fields_by_name['members']._serialized_options = b'\310\336\037\000'
  _MSGCREATEGROUPWITHPOLICY.fields_by_name['decision_policy']._options = None
  _MSGCREATEGROUPWITHPOLICY.fields_by_name['decision_policy']._serialized_options = b'\312\264-\036cosmos.group.v1.DecisionPolicy'
  _MSGCREATEGROUPWITHPOLICY._options = None
  _MSGCREATEGROUPWITHPOLICY._serialized_options = b'\210\240\037\000\202\347\260*\005admin'
  _MSGCREATEGROUPWITHPOLICYRESPONSE.fields_by_name['group_policy_address']._options = None
  _MSGCREATEGROUPWITHPOLICYRESPONSE.fields_by_name['group_policy_address']._serialized_options = b'\322\264-\024cosmos.AddressString'
  _MSGUPDATEGROUPPOLICYDECISIONPOLICY.fields_by_name['admin']._options = None
  _MSGUPDATEGROUPPOLICYDECISIONPOLICY.fields_by_name['admin']._serialized_options = b'\322\264-\024cosmos.AddressString'
  _MSGUPDATEGROUPPOLICYDECISIONPOLICY.fields_by_name['group_policy_address']._options = None
  _MSGUPDATEGROUPPOLICYDECISIONPOLICY.fields_by_name['group_policy_address']._serialized_options = b'\322\264-\024cosmos.AddressString'
  _MSGUPDATEGROUPPOLICYDECISIONPOLICY.fields_by_name['decision_policy']._options = None
  _MSGUPDATEGROUPPOLICYDECISIONPOLICY.fields_by_name['decision_policy']._serialized_options = b'\312\264-\036cosmos.group.v1.DecisionPolicy'
  _MSGUPDATEGROUPPOLICYDECISIONPOLICY._options = None
  _MSGUPDATEGROUPPOLICYDECISIONPOLICY._serialized_options = b'\210\240\037\000\202\347\260*\005admin'
  _MSGUPDATEGROUPPOLICYMETADATA.fields_by_name['admin']._options = None
  _MSGUPDATEGROUPPOLICYMETADATA.fields_by_name['admin']._serialized_options = b'\322\264-\024cosmos.AddressString'
  _MSGUPDATEGROUPPOLICYMETADATA.fields_by_name['group_policy_address']._options = None
  _MSGUPDATEGROUPPOLICYMETADATA.fields_by_name['group_policy_address']._serialized_options = b'\322\264-\024cosmos.AddressString'
  _MSGUPDATEGROUPPOLICYMETADATA._options = None
  _MSGUPDATEGROUPPOLICYMETADATA._serialized_options = b'\202\347\260*\005admin'
  _MSGSUBMITPROPOSAL.fields_by_name['group_policy_address']._options = None
  _MSGSUBMITPROPOSAL.fields_by_name['group_policy_address']._serialized_options = b'\322\264-\024cosmos.AddressString'
  _MSGSUBMITPROPOSAL._options = None
  _MSGSUBMITPROPOSAL._serialized_options = b'\210\240\037\000\202\347\260*\tproposers'
  _MSGWITHDRAWPROPOSAL.fields_by_name['address']._options = None
  _MSGWITHDRAWPROPOSAL.fields_by_name['address']._serialized_options = b'\322\264-\024cosmos.AddressString'
  _MSGWITHDRAWPROPOSAL._options = None
  _MSGWITHDRAWPROPOSAL._serialized_options = b'\202\347\260*\007address'
  _MSGVOTE.fields_by_name['voter']._options = None
  _MSGVOTE.fields_by_name['voter']._serialized_options = b'\322\264-\024cosmos.AddressString'
  _MSGVOTE._options = None
  _MSGVOTE._serialized_options = b'\202\347\260*\005voter'
  _MSGEXEC.fields_by_name['executor']._options = None
  _MSGEXEC.fields_by_name['executor']._serialized_options = b'\322\264-\024cosmos.AddressString'
  _MSGEXEC._options = None
  _MSGEXEC._serialized_options = b'\202\347\260*\006signer'
  _MSGLEAVEGROUP.fields_by_name['address']._options = None
  _MSGLEAVEGROUP.fields_by_name['address']._serialized_options = b'\322\264-\024cosmos.AddressString'
  _MSGLEAVEGROUP._options = None
  _MSGLEAVEGROUP._serialized_options = b'\202\347\260*\007address'
  _globals['_EXEC']._serialized_start=3155
  _globals['_EXEC']._serialized_end=3197
  _globals['_MSGCREATEGROUP']._serialized_start=176
  _globals['_MSGCREATEGROUP']._serialized_end=318
  _globals['_MSGCREATEGROUPRESPONSE']._serialized_start=320
  _globals['_MSGCREATEGROUPRESPONSE']._serialized_end=362
  _globals['_MSGUPDATEGROUPMEMBERS']._serialized_start=365
  _globals['_MSGUPDATEGROUPMEMBERS']._serialized_end=521
  _globals['_MSGUPDATEGROUPMEMBERSRESPONSE']._serialized_start=523
  _globals['_MSGUPDATEGROUPMEMBERSRESPONSE']._serialized_end=554
  _globals['_MSGUPDATEGROUPADMIN']._serialized_start=557
  _globals['_MSGUPDATEGROUPADMIN']._serialized_end=694
  _globals['_MSGUPDATEGROUPADMINRESPONSE']._serialized_start=696
  _globals['_MSGUPDATEGROUPADMINRESPONSE']._serialized_end=725
  _globals['_MSGUPDATEGROUPMETADATA']._serialized_start=727
  _globals['_MSGUPDATEGROUPMETADATA']._serialized_end=840
  _globals['_MSGUPDATEGROUPMETADATARESPONSE']._serialized_start=842
  _globals['_MSGUPDATEGROUPMETADATARESPONSE']._serialized_end=874
  _globals['_MSGCREATEGROUPPOLICY']._serialized_start=877
  _globals['_MSGCREATEGROUPPOLICY']._serialized_end=1075
  _globals['_MSGCREATEGROUPPOLICYRESPONSE']._serialized_start=1077
  _globals['_MSGCREATEGROUPPOLICYRESPONSE']._serialized_end=1150
  _globals['_MSGUPDATEGROUPPOLICYADMIN']._serialized_start=1153
  _globals['_MSGUPDATEGROUPPOLICYADMIN']._serialized_end=1334
  _globals['_MSGCREATEGROUPWITHPOLICY']._serialized_start=1337
  _globals['_MSGCREATEGROUPWITHPOLICY']._serialized_end=1644
  _globals['_MSGCREATEGROUPWITHPOLICYRESPONSE']._serialized_start=1646
  _globals['_MSGCREATEGROUPWITHPOLICYRESPONSE']._serialized_end=1754
  _globals['_MSGUPDATEGROUPPOLICYADMINRESPONSE']._serialized_start=1756
  _globals['_MSGUPDATEGROUPPOLICYADMINRESPONSE']._serialized_end=1791
  _globals['_MSGUPDATEGROUPPOLICYDECISIONPOLICY']._serialized_start=1794
  _globals['_MSGUPDATEGROUPPOLICYDECISIONPOLICY']._serialized_end=2026
  _globals['_MSGUPDATEGROUPPOLICYDECISIONPOLICYRESPONSE']._serialized_start=2028
  _globals['_MSGUPDATEGROUPPOLICYDECISIONPOLICYRESPONSE']._serialized_end=2072
  _globals['_MSGUPDATEGROUPPOLICYMETADATA']._serialized_start=2075
  _globals['_MSGUPDATEGROUPPOLICYMETADATA']._serialized_end=2232
  _globals['_MSGUPDATEGROUPPOLICYMETADATARESPONSE']._serialized_start=2234
  _globals['_MSGUPDATEGROUPPOLICYMETADATARESPONSE']._serialized_end=2272
  _globals['_MSGSUBMITPROPOSAL']._serialized_start=2275
  _globals['_MSGSUBMITPROPOSAL']._serialized_end=2484
  _globals['_MSGSUBMITPROPOSALRESPONSE']._serialized_start=2486
  _globals['_MSGSUBMITPROPOSALRESPONSE']._serialized_end=2534
  _globals['_MSGWITHDRAWPROPOSAL']._serialized_start=2536
  _globals['_MSGWITHDRAWPROPOSAL']._serialized_end=2635
  _globals['_MSGWITHDRAWPROPOSALRESPONSE']._serialized_start=2637
  _globals['_MSGWITHDRAWPROPOSALRESPONSE']._serialized_end=2666
  _globals['_MSGVOTE']._serialized_start=2669
  _globals['_MSGVOTE']._serialized_end=2852
  _globals['_MSGVOTERESPONSE']._serialized_start=2854
  _globals['_MSGVOTERESPONSE']._serialized_end=2871
  _globals['_MSGEXEC']._serialized_start=2873
  _globals['_MSGEXEC']._serialized_end=2960
  _globals['_MSGEXECRESPONSE']._serialized_start=2962
  _globals['_MSGEXECRESPONSE']._serialized_end=3036
  _globals['_MSGLEAVEGROUP']._serialized_start=3038
  _globals['_MSGLEAVEGROUP']._serialized_end=3128
  _globals['_MSGLEAVEGROUPRESPONSE']._serialized_start=3130
  _globals['_MSGLEAVEGROUPRESPONSE']._serialized_end=3153
  _globals['_MSG']._serialized_start=3200
  _globals['_MSG']._serialized_end=4675
# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: fx/gov/v1/tx.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from fxsdk.x.cosmos.msg.v1 import msg_pb2 as cosmos_dot_msg_dot_v1_dot_msg__pb2
from fxsdk.x.cosmos_proto import cosmos_pb2 as cosmos__proto_dot_cosmos__pb2
from fxsdk.x.fx.gov.v1 import params_pb2 as fx_dot_gov_dot_v1_dot_params__pb2
from fxsdk.x.gogoproto import gogo_pb2 as gogoproto_dot_gogo__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x12\x66x/gov/v1/tx.proto\x12\tfx.gov.v1\x1a\x17\x63osmos/msg/v1/msg.proto\x1a\x19\x63osmos_proto/cosmos.proto\x1a\x16\x66x/gov/v1/params.proto\x1a\x14gogoproto/gogo.proto\"w\n\x0fMsgUpdateParams\x12+\n\tauthority\x18\x01 \x01(\tB\x18\xd2\xb4-\x14\x63osmos.AddressString\x12\'\n\x06params\x18\x02 \x01(\x0b\x32\x11.fx.gov.v1.ParamsB\x04\xc8\xde\x1f\x00:\x0e\x82\xe7\xb0*\tauthority\"\x19\n\x17MsgUpdateParamsResponse\"}\n\x12MsgUpdateEGFParams\x12+\n\tauthority\x18\x01 \x01(\tB\x18\xd2\xb4-\x14\x63osmos.AddressString\x12*\n\x06params\x18\x02 \x01(\x0b\x32\x14.fx.gov.v1.EGFParamsB\x04\xc8\xde\x1f\x00:\x0e\x82\xe7\xb0*\tauthority\"\x1c\n\x1aMsgUpdateEGFParamsResponse2\xae\x01\n\x03Msg\x12N\n\x0cUpdateParams\x12\x1a.fx.gov.v1.MsgUpdateParams\x1a\".fx.gov.v1.MsgUpdateParamsResponse\x12W\n\x0fUpdateEGFParams\x12\x1d.fx.gov.v1.MsgUpdateEGFParams\x1a%.fx.gov.v1.MsgUpdateEGFParamsResponseB*Z(github.com/functionx/fx-core/x/gov/typesb\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'fx.gov.v1.tx_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:
  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'Z(github.com/functionx/fx-core/x/gov/types'
  _MSGUPDATEPARAMS.fields_by_name['authority']._options = None
  _MSGUPDATEPARAMS.fields_by_name['authority']._serialized_options = b'\322\264-\024cosmos.AddressString'
  _MSGUPDATEPARAMS.fields_by_name['params']._options = None
  _MSGUPDATEPARAMS.fields_by_name['params']._serialized_options = b'\310\336\037\000'
  _MSGUPDATEPARAMS._options = None
  _MSGUPDATEPARAMS._serialized_options = b'\202\347\260*\tauthority'
  _MSGUPDATEEGFPARAMS.fields_by_name['authority']._options = None
  _MSGUPDATEEGFPARAMS.fields_by_name['authority']._serialized_options = b'\322\264-\024cosmos.AddressString'
  _MSGUPDATEEGFPARAMS.fields_by_name['params']._options = None
  _MSGUPDATEEGFPARAMS.fields_by_name['params']._serialized_options = b'\310\336\037\000'
  _MSGUPDATEEGFPARAMS._options = None
  _MSGUPDATEEGFPARAMS._serialized_options = b'\202\347\260*\tauthority'
  _globals['_MSGUPDATEPARAMS']._serialized_start=131
  _globals['_MSGUPDATEPARAMS']._serialized_end=250
  _globals['_MSGUPDATEPARAMSRESPONSE']._serialized_start=252
  _globals['_MSGUPDATEPARAMSRESPONSE']._serialized_end=277
  _globals['_MSGUPDATEEGFPARAMS']._serialized_start=279
  _globals['_MSGUPDATEEGFPARAMS']._serialized_end=404
  _globals['_MSGUPDATEEGFPARAMSRESPONSE']._serialized_start=406
  _globals['_MSGUPDATEEGFPARAMSRESPONSE']._serialized_end=434
  _globals['_MSG']._serialized_start=437
  _globals['_MSG']._serialized_end=611
# @@protoc_insertion_point(module_scope)



import asyncio
import itertools
import logging

import aiohttp
import requests
import ujson

from datetime import datetime
from typing import Optional, Dict
from requests.sessions import HTTPAdapter
from urllib3.util.retry import Retry
from collections import OrderedDict

from fxsdk.x.cosmos.tx.v1beta1.service_pb2 import BroadcastMode, BROADCAST_MODE_SYNC, BROADCAST_MODE_ASYNC, \
    BROADCAST_MODE_BLOCK
from fxsdk.x.cosmos.tx.v1beta1.tx_pb2 import Tx, TxRaw

requests.models.json = ujson


def _get_headers():
    return {
        'Accept': 'application/json',
        'Content-Type': 'application/json',
        'User-Agent': 'fx-py-sdk',
    }


class RPCException(Exception):
    def __init__(self, response):
        self.code = 0
        try:
            json_res = ujson.loads(response.content)
        except ValueError:
            self.message = 'Invalid JSON error message from Binance Chain: {}'.format(response.text)
        else:
            self.code = json_res['error']['code']
            self.message = json_res['error']['message']
        self.status_code = response.status_code
        self.response = response
        self.request = getattr(response, 'request', None)

    def __str__(self):  # pragma: no cover
        return f'RPCError(code={self.code}): {self.message}'


class RpcRequest:

    def __init__(self, method, req_id, params=None):
        self._method = method
        self._params = params
        self._id = req_id

    @staticmethod
    def _sort_request(request):
        sort_order = ["jsonrpc", "method", "params", "id"]
        return OrderedDict(sorted(request.items(), key=lambda k: sort_order.index(k[0])))

    def __str__(self):
        request = {
            'jsonrpc': '2.0',
            'method': self._method,
            'id': self._id
        }

        if self._params:
            request['params'] = self._params

        return ujson.dumps(self._sort_request(request), ensure_ascii=False)


class BaseRpcClient:
    id_generator = itertools.count(1)

    def __init__(self, endpoint_url, requests_params: Optional[Dict] = None):
        self._endpoint_url = endpoint_url
        self._requests_params = requests_params
        self._log = logging.getLogger(__name__)
        self.session = self._init_session()

    @staticmethod
    def _init_session():
        session = requests.session()
        session.headers.update(_get_headers())
        return session

    def _get_rpc_request(self, path, **kwargs) -> str:
        rpc_request = RpcRequest(path, next(self.id_generator), kwargs.get('data', None))
        return str(rpc_request)


class HttpRpcClient(BaseRpcClient):
    __doc__ = """
    https://docs.cometbft.com/main/rpc/
    """

    def __init__(self, endpoint_url, requests_params: Optional[Dict] = None, max_retries=3):
        super(HttpRpcClient, self).__init__(endpoint_url, requests_params)
        if max_retries:
            retries = Retry(total=max_retries,
                            backoff_factor=.5,
                            status_forcelist=[500, 502, 503, 504])
            self.session.mount('http://', HTTPAdapter(max_retries=retries))

    def _request(self, path, **kwargs) -> Dict:
        rpc_request = self._get_rpc_request(path, **kwargs)
        response = self.session.post(self._endpoint_url, data=rpc_request.encode(), headers=_get_headers())
        return self._handle_response(response)

    @staticmethod
    def _handle_response(response) -> Dict:
        try:
            res = response.json()

            if 'error' in res and res['error']:
                raise RPCException(response)

            # by default return full response
            # if it's a normal response we have a data attribute, return that
            if 'result' in res:
                res = res['result']
            return res
        except ValueError:
            raise RPCException('Invalid Response: %s' % response.text)

    def avg_block_time_interval(self, block_interval: int = 2000) -> float:
        latest_block = self.get_block()
        latest_block_time = latest_block.get('block', {}).get('header', {}).get('time', None)
        latest_block_height = int(latest_block.get('block', {}).get('header', {}).get('height', None))
        if latest_block_height > block_interval:
            block = self.get_block(latest_block_height - block_interval)
        else:
            block = self.get_block(1)
            block_interval = latest_block_height - 1
        block_time = block.get('block', {}).get('header', {}).get('time', None)
        latest_block_time = datetime.fromisoformat(latest_block_time)
        block_time = datetime.fromisoformat(block_time)
        block_time_interval = (latest_block_time.timestamp() - block_time.timestamp()) / block_interval
        return float(format(block_time_interval, '.3f'))

    def get_abci_info(self) -> Dict:
        return self._request('abci_info')

    def get_consensus_state(self) -> Dict:
        return self._request('consensus_state')

    def dump_consensus_state(self) -> Dict:
        return self._request('dump_consensus_state')

    def get_genesis(self) -> Dict:
        return self._request('genesis')

    def get_net_info(self) -> Dict:
        return self._request('net_info')

    def get_num_unconfirmed_txs(self) -> Dict:
        return self._request('num_unconfirmed_txs')

    def get_unconfirmed_txs(self) -> Dict:
        return self._request('unconfirmed_txs')

    def get_status(self) -> Dict:
        return self._request('status')

    def get_health(self) -> Dict:
        return self._request('health')

    def get_validators(self, height: int, page: int = 1) -> Dict:
        data = {
            'height': str(height),
            'page': str(page),
            'per_page': '30'
        }
        return self._request('validators', data=data)

    def get_consensus_params(self, height: Optional[int] = None) -> Dict:
        data = {
            'height': str(height) if height else None
        }

        return self._request('consensus_params', data=data)

    def abci_query(self, data: str,
                   path: Optional[str] = None,
                   prove: Optional[bool] = None,
                   height: Optional[int] = None) -> Dict:
        data = {
            'data': data
        }
        if path:
            data['path'] = path
        if prove:
            data['prove'] = str(prove)
        if height:
            data['height'] = str(height)

        return self._request('abci_query', data=data)

    def get_block_header(self, height: Optional[int] = None) -> Dict:
        data = {
            'height': str(height) if height else None
        }

        return self._request('header', data=data)

    def get_block(self, height: Optional[int] = None) -> Dict:
        data = {
            'height': str(height) if height else None
        }

        return self._request('block', data=data)

    def block_search(self, query: str, prove: Optional[bool] = None,
                     page: Optional[int] = None, per_page: Optional[int] = None,
                     order_by: Optional[str] = None) -> Dict:
        data = {
            'query': query
        }
        if prove:
            data['prove'] = str(prove)
        if page:
            data['page'] = str(page)
        if per_page:
            data['per_page'] = str(per_page)
        if order_by:
            data['order_by'] = order_by

        return self._request('block_search', data=data)

    def get_block_results(self, height: int) -> Dict:
        data = {
            'height': str(height)
        }

        return self._request('block_results', data=data)

    def get_block_commit(self, height: int) -> Dict:
        data = {
            'height': str(height)
        }

        return self._request('commit', data=data)

    def get_blockchain_info(self, min_height: int, max_height: int) -> Dict:
        assert max_height > min_height

        data = {
            'minHeight': str(min_height),
            'maxHeight': str(max_height)
        }

        return self._request('blockchain', data=data)

    def check_tx(self, tx: Tx) -> Dict:
        tx_raw = TxRaw(body_bytes=tx.body.SerializeToString(),
                       auth_info_bytes=tx.auth_info.SerializeToString(),
                       signatures=tx.signatures)
        tx_bytes = tx_raw.SerializeToString()
        data = {
            'tx': '0x' + tx_bytes.hex(),
        }

        return self._request('check_tx', data=data)

    def broadcast_tx(self, tx: Tx, mode: BroadcastMode = BROADCAST_MODE_SYNC) -> Dict:
        tx_raw = TxRaw(body_bytes=tx.body.SerializeToString(),
                       auth_info_bytes=tx.auth_info.SerializeToString(),
                       signatures=tx.signatures)
        tx_bytes = tx_raw.SerializeToString()
        data = {
            'tx': '0x' + tx_bytes.hex(),
        }
        if mode == BROADCAST_MODE_SYNC:
            return self._broadcast_tx_sync(data)
        elif mode == BROADCAST_MODE_ASYNC:
            return self._broadcast_tx_async(data)
        elif mode == BROADCAST_MODE_BLOCK:
            return self._broadcast_tx_commit(data)

    def _broadcast_tx_async(self, data: Dict) -> Dict:
        return self._request("broadcast_tx_async", data=data)

    def _broadcast_tx_commit(self, data: Dict) -> Dict:
        return self._request("broadcast_tx_commit", data=data)

    def _broadcast_tx_sync(self, data: Dict) -> Dict:
        return self._request("broadcast_tx_sync", data=data)

    def get_tx(self, tx_hash: str, prove: Optional[bool] = None) -> Dict:
        data = {
            'hash': tx_hash
        }
        if prove:
            data['prove'] = str(prove)

        return self._request('tx', data=data)

    def tx_search(self, query: str, prove: Optional[bool] = None,
                  page: Optional[int] = None, per_page: Optional[int] = None,
                  order_by: Optional[str] = None) -> Dict:
        data = {
            'query': query
        }
        if prove:
            data['prove'] = str(prove)
        if page:
            data['page'] = str(page)
        if per_page:
            data['per_page'] = str(per_page)
        if order_by:
            data['order_by'] = order_by

        return self._request('tx_search', data=data)


class AsyncHttpRpcClient(BaseRpcClient):
    __doc__ = HttpRpcClient.__doc__

    @classmethod
    async def create(cls, endpoint_url) -> 'AsyncHttpRpcClient':
        return AsyncHttpRpcClient(endpoint_url)

    def _init_session(self, **kwargs):
        loop = kwargs.get('loop', asyncio.get_event_loop())
        session = aiohttp.ClientSession(
            loop=loop,
            headers=_get_headers(),
            json_serialize=ujson.dumps
        )
        return session

    async def _request(self, path, **kwargs) -> Dict:
        rpc_request = self._get_rpc_request(path, **kwargs)
        response = await self.session.post(self._endpoint_url, data=rpc_request.encode(), headers=_get_headers())
        return await self._handle_response(response)

    @staticmethod
    async def _handle_response(response) -> Dict:
        try:
            res = await response.json()

            if 'error' in res and res['error']:
                raise RPCException(response)

            # by default return full response
            # if it's a normal response we have a data attribute, return that
            if 'result' in res:
                res = res['result']
            return res
        except ValueError:
            raise RPCException('Invalid Response: %s' % await response.text())

    async def get_abci_info(self) -> Dict:
        return await self._request('abci_info')

    async def get_consensus_state(self) -> Dict:
        return await self._request('consensus_state')

    async def dump_consensus_state(self) -> Dict:
        return await self._request('dump_consensus_state')

    async def get_genesis(self) -> Dict:
        return await self._request('genesis')

    async def get_net_info(self) -> Dict:
        return await self._request('net_info')

    async def get_num_unconfirmed_txs(self) -> Dict:
        return await self._request('num_unconfirmed_txs')

    async def get_status(self) -> Dict:
        return await self._request('status')

    async def get_health(self) -> Dict:
        return await self._request('health')

    async def get_unconfirmed_txs(self) -> Dict:
        return await self._request('unconfirmed_txs')

    async def get_validators(self, height: int, page: int = 1) -> Dict:
        data = {
            'height': str(height),
            'page': str(page),
            'per_page': '30'
        }
        return await self._request('validators', data=data)

    async def abci_query(self, data: str, path: Optional[str] = None,
                         prove: Optional[bool] = None, height: Optional[int] = None) -> Dict:
        data = {
            'data': data
        }
        if path:
            data['path'] = path
        if prove:
            data['prove'] = str(prove)
        if height:
            data['height'] = str(height)

        return await self._request('abci_query', data=data)

    async def get_block_header(self, height: Optional[int] = None) -> Dict:
        data = {
            'height': str(height) if height else None
        }
        return await self._request('header', data=data)

    async def get_block(self, height: Optional[int] = None) -> Dict:
        data = {
            'height': str(height) if height else None
        }
        return await self._request('block', data=data)

    async def block_search(self, query: str, prove: Optional[bool] = None,
                           page: Optional[int] = None, per_page: Optional[int] = None,
                           order_by: Optional[str] = None) -> Dict:
        data = {
            'query': query
        }
        if prove:
            data['prove'] = str(prove)
        if page:
            data['page'] = str(page)
        if per_page:
            data['per_page'] = str(per_page)
        if order_by:
            data['order_by'] = order_by

        return await self._request('block_search', data=data)

    async def get_block_results(self, height: int) -> Dict:
        data = {
            'height': str(height)
        }
        return await self._request('block_results', data=data)

    async def get_block_commit(self, height: int) -> Dict:
        data = {
            'height': str(height)
        }
        return await self._request('commit', data=data)

    async def get_blockchain_info(self, min_height: int, max_height: int) -> Dict:
        assert max_height > min_height

        data = {
            'minHeight': str(min_height),
            'maxHeight': str(max_height)
        }

        return await self._request('blockchain', data=data)

    async def check_tx(self, tx: Tx) -> Dict:
        tx_raw = TxRaw(body_bytes=tx.body.SerializeToString(),
                       auth_info_bytes=tx.auth_info.SerializeToString(),
                       signatures=tx.signatures)
        tx_bytes = tx_raw.SerializeToString()
        data = {
            'tx': '0x' + tx_bytes.hex(),
        }

        return await self._request('check_tx', data=data)

    async def broadcast_tx(self, tx: Tx, mode: BroadcastMode = BROADCAST_MODE_SYNC) -> Dict:
        tx_raw = TxRaw(body_bytes=tx.body.SerializeToString(),
                       auth_info_bytes=tx.auth_info.SerializeToString(),
                       signatures=tx.signatures)
        tx_bytes = tx_raw.SerializeToString()
        data = {
            'tx': '0x' + tx_bytes.hex(),
        }
        if mode == BROADCAST_MODE_SYNC:
            return await self._broadcast_tx_sync(data)
        elif mode == BROADCAST_MODE_ASYNC:
            return await self._broadcast_tx_async(data)
        elif mode == BROADCAST_MODE_BLOCK:
            return await self._broadcast_tx_commit(data)

    async def _broadcast_tx_async(self, data: Dict) -> Dict:
        return await self._request('broadcast_tx_async', data=data)

    async def _broadcast_tx_commit(self, data: Dict) -> Dict:
        return await self._request('broadcast_tx_commit', data=data)

    async def _broadcast_tx_sync(self, data: Dict) -> Dict:
        return await self._request('broadcast_tx_sync', data=data)

    async def get_consensus_params(self, height: Optional[int] = None) -> Dict:
        data = {
            'height': str(height) if height else None
        }

        return await self._request('consensus_params', data=data)

    async def get_tx(self, tx_hash: str, prove: Optional[bool] = None) -> Dict:
        data = {
            'hash': tx_hash
        }
        if prove:
            data['prove'] = str(prove)

        return await self._request('tx', data=data)

    async def tx_search(self, query: str, prove: Optional[bool] = None,
                        page: Optional[int] = None, per_page: Optional[int] = None,
                        order_by: Optional[str] = None) -> Dict:
        data = {
            'query': query
        }
        if prove:
            data['prove'] = str(prove)
        if page:
            data['page'] = str(page)
        if per_page:
            data['per_page'] = str(per_page)
        if order_by:
            data['order_by'] = order_by

        return await self._request('tx_search', data=data)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: fx/crosschain/v1/query.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from fxsdk.x.cosmos.base.v1beta1 import coin_pb2 as cosmos_dot_base_dot_v1beta1_dot_coin__pb2
from fxsdk.x.fx.crosschain.v1 import tx_pb2 as fx_dot_crosschain_dot_v1_dot_tx__pb2
from fxsdk.x.fx.crosschain.v1 import types_pb2 as fx_dot_crosschain_dot_v1_dot_types__pb2
from fxsdk.x.gogoproto import gogo_pb2 as gogoproto_dot_gogo__pb2
from google.api import annotations_pb2 as google_dot_api_dot_annotations__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x1c\x66x/crosschain/v1/query.proto\x12\x18\x66x.gravity.crosschain.v1\x1a\x1e\x63osmos/base/v1beta1/coin.proto\x1a\x19\x66x/crosschain/v1/tx.proto\x1a\x1c\x66x/crosschain/v1/types.proto\x1a\x14gogoproto/gogo.proto\x1a\x1cgoogle/api/annotations.proto\"(\n\x12QueryParamsRequest\x12\x12\n\nchain_name\x18\x01 \x01(\t\"M\n\x13QueryParamsResponse\x12\x36\n\x06params\x18\x01 \x01(\x0b\x32 .fx.gravity.crosschain.v1.ParamsB\x04\xc8\xde\x1f\x00\"2\n\x1cQueryCurrentOracleSetRequest\x12\x12\n\nchain_name\x18\x01 \x01(\t\"X\n\x1dQueryCurrentOracleSetResponse\x12\x37\n\noracle_set\x18\x01 \x01(\x0b\x32#.fx.gravity.crosschain.v1.OracleSet\"A\n\x1cQueryOracleSetRequestRequest\x12\x12\n\nchain_name\x18\x01 \x01(\t\x12\r\n\x05nonce\x18\x02 \x01(\x04\"X\n\x1dQueryOracleSetRequestResponse\x12\x37\n\noracle_set\x18\x01 \x01(\x0b\x32#.fx.gravity.crosschain.v1.OracleSet\"Z\n\x1cQueryOracleSetConfirmRequest\x12\x12\n\nchain_name\x18\x01 \x01(\t\x12\x17\n\x0f\x62ridger_address\x18\x02 \x01(\t\x12\r\n\x05nonce\x18\x03 \x01(\x04\"_\n\x1dQueryOracleSetConfirmResponse\x12>\n\x07\x63onfirm\x18\x01 \x01(\x0b\x32-.fx.gravity.crosschain.v1.MsgOracleSetConfirm\"I\n$QueryOracleSetConfirmsByNonceRequest\x12\x12\n\nchain_name\x18\x01 \x01(\t\x12\r\n\x05nonce\x18\x02 \x01(\x04\"h\n%QueryOracleSetConfirmsByNonceResponse\x12?\n\x08\x63onfirms\x18\x01 \x03(\x0b\x32-.fx.gravity.crosschain.v1.MsgOracleSetConfirm\"7\n!QueryLastOracleSetRequestsRequest\x12\x12\n\nchain_name\x18\x01 \x01(\t\"^\n\"QueryLastOracleSetRequestsResponse\x12\x38\n\x0boracle_sets\x18\x01 \x03(\x0b\x32#.fx.gravity.crosschain.v1.OracleSet\"\\\n-QueryLastPendingOracleSetRequestByAddrRequest\x12\x12\n\nchain_name\x18\x01 \x01(\t\x12\x17\n\x0f\x62ridger_address\x18\x02 \x01(\t\"j\n.QueryLastPendingOracleSetRequestByAddrResponse\x12\x38\n\x0boracle_sets\x18\x01 \x03(\x0b\x32#.fx.gravity.crosschain.v1.OracleSet\"m\n\x14QueryBatchFeeRequest\x12\x12\n\nchain_name\x18\x01 \x01(\t\x12\x41\n\x0cminBatchFees\x18\x02 \x03(\x0b\x32%.fx.gravity.crosschain.v1.MinBatchFeeB\x04\xc8\xde\x1f\x00\"P\n\x15QueryBatchFeeResponse\x12\x37\n\nbatch_fees\x18\x01 \x03(\x0b\x32#.fx.gravity.crosschain.v1.BatchFees\"X\n)QueryLastPendingBatchRequestByAddrRequest\x12\x12\n\nchain_name\x18\x01 \x01(\t\x12\x17\n\x0f\x62ridger_address\x18\x02 \x01(\t\"f\n*QueryLastPendingBatchRequestByAddrResponse\x12\x38\n\x05\x62\x61tch\x18\x01 \x01(\x0b\x32).fx.gravity.crosschain.v1.OutgoingTxBatch\"3\n\x1dQueryOutgoingTxBatchesRequest\x12\x12\n\nchain_name\x18\x01 \x01(\t\"\\\n\x1eQueryOutgoingTxBatchesResponse\x12:\n\x07\x62\x61tches\x18\x01 \x03(\x0b\x32).fx.gravity.crosschain.v1.OutgoingTxBatch\"\\\n\x1fQueryBatchRequestByNonceRequest\x12\x12\n\nchain_name\x18\x01 \x01(\t\x12\x16\n\x0etoken_contract\x18\x02 \x01(\t\x12\r\n\x05nonce\x18\x03 \x01(\x04\"\\\n QueryBatchRequestByNonceResponse\x12\x38\n\x05\x62\x61tch\x18\x01 \x01(\x0b\x32).fx.gravity.crosschain.v1.OutgoingTxBatch\"n\n\x18QueryBatchConfirmRequest\x12\x12\n\nchain_name\x18\x01 \x01(\t\x12\x16\n\x0etoken_contract\x18\x02 \x01(\t\x12\x17\n\x0f\x62ridger_address\x18\x03 \x01(\t\x12\r\n\x05nonce\x18\x04 \x01(\x04\"W\n\x19QueryBatchConfirmResponse\x12:\n\x07\x63onfirm\x18\x01 \x01(\x0b\x32).fx.gravity.crosschain.v1.MsgConfirmBatch\"V\n\x19QueryBatchConfirmsRequest\x12\x12\n\nchain_name\x18\x01 \x01(\t\x12\x16\n\x0etoken_contract\x18\x02 \x01(\t\x12\r\n\x05nonce\x18\x03 \x01(\x04\"Y\n\x1aQueryBatchConfirmsResponse\x12;\n\x08\x63onfirms\x18\x01 \x03(\x0b\x32).fx.gravity.crosschain.v1.MsgConfirmBatch\"O\n QueryLastEventNonceByAddrRequest\x12\x12\n\nchain_name\x18\x01 \x01(\t\x12\x17\n\x0f\x62ridger_address\x18\x02 \x01(\t\"8\n!QueryLastEventNonceByAddrResponse\x12\x13\n\x0b\x65vent_nonce\x18\x01 \x01(\x04\"=\n\x18QueryTokenToDenomRequest\x12\x12\n\nchain_name\x18\x01 \x01(\t\x12\r\n\x05token\x18\x02 \x01(\t\"*\n\x19QueryTokenToDenomResponse\x12\r\n\x05\x64\x65nom\x18\x01 \x01(\t\"=\n\x18QueryDenomToTokenRequest\x12\x12\n\nchain_name\x18\x01 \x01(\t\x12\r\n\x05\x64\x65nom\x18\x02 \x01(\t\"*\n\x19QueryDenomToTokenResponse\x12\r\n\x05token\x18\x01 \x01(\t\"F\n\x18QueryOracleByAddrRequest\x12\x12\n\nchain_name\x18\x01 \x01(\t\x12\x16\n\x0eoracle_address\x18\x02 \x01(\t\"G\n\x13QueryOracleResponse\x12\x30\n\x06oracle\x18\x01 \x01(\x0b\x32 .fx.gravity.crosschain.v1.Oracle\"P\n QueryOracleByExternalAddrRequest\x12\x12\n\nchain_name\x18\x01 \x01(\t\x12\x18\n\x10\x65xternal_address\x18\x02 \x01(\t\"N\n\x1fQueryOracleByBridgerAddrRequest\x12\x12\n\nchain_name\x18\x01 \x01(\t\x12\x17\n\x0f\x62ridger_address\x18\x02 \x01(\t\"O\n!QueryPendingSendToExternalRequest\x12\x12\n\nchain_name\x18\x01 \x01(\t\x12\x16\n\x0esender_address\x18\x02 \x01(\t\"\xbb\x01\n\"QueryPendingSendToExternalResponse\x12J\n\x14transfers_in_batches\x18\x01 \x03(\x0b\x32,.fx.gravity.crosschain.v1.OutgoingTransferTx\x12I\n\x13unbatched_transfers\x18\x02 \x03(\x0b\x32,.fx.gravity.crosschain.v1.OutgoingTransferTx\"9\n#QueryLastObservedBlockHeightRequest\x12\x12\n\nchain_name\x18\x01 \x01(\t\"[\n$QueryLastObservedBlockHeightResponse\x12\x1d\n\x15\x65xternal_block_height\x18\x01 \x01(\x04\x12\x14\n\x0c\x62lock_height\x18\x02 \x01(\x04\"U\n&QueryLastEventBlockHeightByAddrRequest\x12\x12\n\nchain_name\x18\x01 \x01(\t\x12\x17\n\x0f\x62ridger_address\x18\x02 \x01(\t\"?\n\'QueryLastEventBlockHeightByAddrResponse\x12\x14\n\x0c\x62lock_height\x18\x01 \x01(\x04\")\n\x13QueryOraclesRequest\x12\x12\n\nchain_name\x18\x01 \x01(\t\"O\n\x14QueryOraclesResponse\x12\x37\n\x07oracles\x18\x01 \x03(\x0b\x32 .fx.gravity.crosschain.v1.OracleB\x04\xc8\xde\x1f\x00\"=\n\'QueryProjectedBatchTimeoutHeightRequest\x12\x12\n\nchain_name\x18\x01 \x01(\t\"B\n(QueryProjectedBatchTimeoutHeightResponse\x12\x16\n\x0etimeout_height\x18\x01 \x01(\x04\".\n\x18QueryBridgeTokensRequest\x12\x12\n\nchain_name\x18\x01 \x01(\t\"Y\n\x19QueryBridgeTokensResponse\x12<\n\rbridge_tokens\x18\x01 \x03(\x0b\x32%.fx.gravity.crosschain.v1.BridgeToken\"B\n\x1dQueryBridgeCoinByDenomRequest\x12\x12\n\nchain_name\x18\x01 \x01(\t\x12\r\n\x05\x64\x65nom\x18\x02 \x01(\t\"O\n\x1eQueryBridgeCoinByDenomResponse\x12-\n\x04\x63oin\x18\x01 \x01(\x0b\x32\x19.cosmos.base.v1beta1.CoinB\x04\xc8\xde\x1f\x00\"\x1d\n\x1bQueryBridgeChainListRequest\"3\n\x1cQueryBridgeChainListResponse\x12\x13\n\x0b\x63hain_names\x18\x01 \x03(\t2\xaf&\n\x05Query\x12\x87\x01\n\x06Params\x12,.fx.gravity.crosschain.v1.QueryParamsRequest\x1a-.fx.gravity.crosschain.v1.QueryParamsResponse\" \x82\xd3\xe4\x93\x02\x1a\x12\x18/fx/crosschain/v1/params\x12\xb1\x01\n\x10\x43urrentOracleSet\x12\x36.fx.gravity.crosschain.v1.QueryCurrentOracleSetRequest\x1a\x37.fx.gravity.crosschain.v1.QueryCurrentOracleSetResponse\",\x82\xd3\xe4\x93\x02&\x12$/fx/crosschain/v1/oracle_set/current\x12\xb1\x01\n\x10OracleSetRequest\x12\x36.fx.gravity.crosschain.v1.QueryOracleSetRequestRequest\x1a\x37.fx.gravity.crosschain.v1.QueryOracleSetRequestResponse\",\x82\xd3\xe4\x93\x02&\x12$/fx/crosschain/v1/oracle_set/request\x12\xb1\x01\n\x10OracleSetConfirm\x12\x36.fx.gravity.crosschain.v1.QueryOracleSetConfirmRequest\x1a\x37.fx.gravity.crosschain.v1.QueryOracleSetConfirmResponse\",\x82\xd3\xe4\x93\x02&\x12$/fx/crosschain/v1/oracle_set/confirm\x12\xca\x01\n\x18OracleSetConfirmsByNonce\x12>.fx.gravity.crosschain.v1.QueryOracleSetConfirmsByNonceRequest\x1a?.fx.gravity.crosschain.v1.QueryOracleSetConfirmsByNonceResponse\"-\x82\xd3\xe4\x93\x02\'\x12%/fx/crosschain/v1/oracle_set/confirms\x12\xc1\x01\n\x15LastOracleSetRequests\x12;.fx.gravity.crosschain.v1.QueryLastOracleSetRequestsRequest\x1a<.fx.gravity.crosschain.v1.QueryLastOracleSetRequestsResponse\"-\x82\xd3\xe4\x93\x02\'\x12%/fx/crosschain/v1/oracle_set/requests\x12\xe1\x01\n!LastPendingOracleSetRequestByAddr\x12G.fx.gravity.crosschain.v1.QueryLastPendingOracleSetRequestByAddrRequest\x1aH.fx.gravity.crosschain.v1.QueryLastPendingOracleSetRequestByAddrResponse\")\x82\xd3\xe4\x93\x02#\x12!/fx/crosschain/v1/oracle_set/last\x12\xd0\x01\n\x1dLastPendingBatchRequestByAddr\x12\x43.fx.gravity.crosschain.v1.QueryLastPendingBatchRequestByAddrRequest\x1a\x44.fx.gravity.crosschain.v1.QueryLastPendingBatchRequestByAddrResponse\"$\x82\xd3\xe4\x93\x02\x1e\x12\x1c/fx/crosschain/v1/batch/last\x12\xbd\x01\n\x14LastEventNonceByAddr\x12:.fx.gravity.crosschain.v1.QueryLastEventNonceByAddrRequest\x1a;.fx.gravity.crosschain.v1.QueryLastEventNonceByAddrResponse\",\x82\xd3\xe4\x93\x02&\x12$/fx/crosschain/v1/oracle/event_nonce\x12\xd6\x01\n\x1aLastEventBlockHeightByAddr\x12@.fx.gravity.crosschain.v1.QueryLastEventBlockHeightByAddrRequest\x1a\x41.fx.gravity.crosschain.v1.QueryLastEventBlockHeightByAddrResponse\"3\x82\xd3\xe4\x93\x02-\x12+/fx/crosschain/v1/oracle/event/block_height\x12\x92\x01\n\tBatchFees\x12..fx.gravity.crosschain.v1.QueryBatchFeeRequest\x1a/.fx.gravity.crosschain.v1.QueryBatchFeeResponse\"$\x82\xd3\xe4\x93\x02\x1e\x12\x1c/fx/crosschain/v1/batch_fees\x12\xc9\x01\n\x17LastObservedBlockHeight\x12=.fx.gravity.crosschain.v1.QueryLastObservedBlockHeightRequest\x1a>.fx.gravity.crosschain.v1.QueryLastObservedBlockHeightResponse\"/\x82\xd3\xe4\x93\x02)\x12\'/fx/crosschain/v1/observed/block_height\x12\xb3\x01\n\x11OutgoingTxBatches\x12\x37.fx.gravity.crosschain.v1.QueryOutgoingTxBatchesRequest\x1a\x38.fx.gravity.crosschain.v1.QueryOutgoingTxBatchesResponse\"+\x82\xd3\xe4\x93\x02%\x12#/fx/crosschain/v1/batch/outgoing_tx\x12\xb5\x01\n\x13\x42\x61tchRequestByNonce\x12\x39.fx.gravity.crosschain.v1.QueryBatchRequestByNonceRequest\x1a:.fx.gravity.crosschain.v1.QueryBatchRequestByNonceResponse\"\'\x82\xd3\xe4\x93\x02!\x12\x1f/fx/crosschain/v1/batch/request\x12\xa0\x01\n\x0c\x42\x61tchConfirm\x12\x32.fx.gravity.crosschain.v1.QueryBatchConfirmRequest\x1a\x33.fx.gravity.crosschain.v1.QueryBatchConfirmResponse\"\'\x82\xd3\xe4\x93\x02!\x12\x1f/fx/crosschain/v1/batch/confirm\x12\xa4\x01\n\rBatchConfirms\x12\x33.fx.gravity.crosschain.v1.QueryBatchConfirmsRequest\x1a\x34.fx.gravity.crosschain.v1.QueryBatchConfirmsResponse\"(\x82\xd3\xe4\x93\x02\"\x12 /fx/crosschain/v1/batch/confirms\x12\x98\x01\n\x0cTokenToDenom\x12\x32.fx.gravity.crosschain.v1.QueryTokenToDenomRequest\x1a\x33.fx.gravity.crosschain.v1.QueryTokenToDenomResponse\"\x1f\x82\xd3\xe4\x93\x02\x19\x12\x17/fx/crosschain/v1/denom\x12\x98\x01\n\x0c\x44\x65nomToToken\x12\x32.fx.gravity.crosschain.v1.QueryDenomToTokenRequest\x1a\x33.fx.gravity.crosschain.v1.QueryDenomToTokenResponse\"\x1f\x82\xd3\xe4\x93\x02\x19\x12\x17/fx/crosschain/v1/token\x12\x9e\x01\n\x0fGetOracleByAddr\x12\x32.fx.gravity.crosschain.v1.QueryOracleByAddrRequest\x1a-.fx.gravity.crosschain.v1.QueryOracleResponse\"(\x82\xd3\xe4\x93\x02\"\x12 /fx/crosschain/v1/oracle_by_addr\x12\xb7\x01\n\x17GetOracleByExternalAddr\x12:.fx.gravity.crosschain.v1.QueryOracleByExternalAddrRequest\x1a-.fx.gravity.crosschain.v1.QueryOracleResponse\"1\x82\xd3\xe4\x93\x02+\x12)/fx/crosschain/v1/oracle_by_external_addr\x12\xb4\x01\n\x16GetOracleByBridgerAddr\x12\x39.fx.gravity.crosschain.v1.QueryOracleByBridgerAddrRequest\x1a-.fx.gravity.crosschain.v1.QueryOracleResponse\"0\x82\xd3\xe4\x93\x02*\x12(/fx/crosschain/v1/oracle_by_bridger_addr\x12\xc9\x01\n\x18GetPendingSendToExternal\x12;.fx.gravity.crosschain.v1.QueryPendingSendToExternalRequest\x1a<.fx.gravity.crosschain.v1.QueryPendingSendToExternalResponse\"2\x82\xd3\xe4\x93\x02,\x12*/fx/crosschain/v1/pending_send_to_external\x12\x8b\x01\n\x07Oracles\x12-.fx.gravity.crosschain.v1.QueryOraclesRequest\x1a..fx.gravity.crosschain.v1.QueryOraclesResponse\"!\x82\xd3\xe4\x93\x02\x1b\x12\x19/fx/crosschain/v1/oracles\x12\xd7\x01\n\x1bProjectedBatchTimeoutHeight\x12\x41.fx.gravity.crosschain.v1.QueryProjectedBatchTimeoutHeightRequest\x1a\x42.fx.gravity.crosschain.v1.QueryProjectedBatchTimeoutHeightResponse\"1\x82\xd3\xe4\x93\x02+\x12)/fx/crosschain/v1/projected_batch_timeout\x12\xa0\x01\n\x0c\x42ridgeTokens\x12\x32.fx.gravity.crosschain.v1.QueryBridgeTokensRequest\x1a\x33.fx.gravity.crosschain.v1.QueryBridgeTokensResponse\"\'\x82\xd3\xe4\x93\x02!\x12\x1f/fx/crosschain/v1/bridge_tokens\x12\xb6\x01\n\x11\x42ridgeCoinByDenom\x12\x37.fx.gravity.crosschain.v1.QueryBridgeCoinByDenomRequest\x1a\x38.fx.gravity.crosschain.v1.QueryBridgeCoinByDenomResponse\".\x82\xd3\xe4\x93\x02(\x12&/fx/crosschain/v1/bridge_coin_by_denom\x12\xad\x01\n\x0f\x42ridgeChainList\x12\x35.fx.gravity.crosschain.v1.QueryBridgeChainListRequest\x1a\x36.fx.gravity.crosschain.v1.QueryBridgeChainListResponse\"+\x82\xd3\xe4\x93\x02%\x12#/fx/crosschain/v1/bridge_chain_listB1Z/github.com/functionx/fx-core/x/crosschain/typesb\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'fx.crosschain.v1.query_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:
  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'Z/github.com/functionx/fx-core/x/crosschain/types'
  _QUERYPARAMSRESPONSE.fields_by_name['params']._options = None
  _QUERYPARAMSRESPONSE.fields_by_name['params']._serialized_options = b'\310\336\037\000'
  _QUERYBATCHFEEREQUEST.fields_by_name['minBatchFees']._options = None
  _QUERYBATCHFEEREQUEST.fields_by_name['minBatchFees']._serialized_options = b'\310\336\037\000'
  _QUERYORACLESRESPONSE.fields_by_name['oracles']._options = None
  _QUERYORACLESRESPONSE.fields_by_name['oracles']._serialized_options = b'\310\336\037\000'
  _QUERYBRIDGECOINBYDENOMRESPONSE.fields_by_name['coin']._options = None
  _QUERYBRIDGECOINBYDENOMRESPONSE.fields_by_name['coin']._serialized_options = b'\310\336\037\000'
  _QUERY.methods_by_name['Params']._options = None
  _QUERY.methods_by_name['Params']._serialized_options = b'\202\323\344\223\002\032\022\030/fx/crosschain/v1/params'
  _QUERY.methods_by_name['CurrentOracleSet']._options = None
  _QUERY.methods_by_name['CurrentOracleSet']._serialized_options = b'\202\323\344\223\002&\022$/fx/crosschain/v1/oracle_set/current'
  _QUERY.methods_by_name['OracleSetRequest']._options = None
  _QUERY.methods_by_name['OracleSetRequest']._serialized_options = b'\202\323\344\223\002&\022$/fx/crosschain/v1/oracle_set/request'
  _QUERY.methods_by_name['OracleSetConfirm']._options = None
  _QUERY.methods_by_name['OracleSetConfirm']._serialized_options = b'\202\323\344\223\002&\022$/fx/crosschain/v1/oracle_set/confirm'
  _QUERY.methods_by_name['OracleSetConfirmsByNonce']._options = None
  _QUERY.methods_by_name['OracleSetConfirmsByNonce']._serialized_options = b'\202\323\344\223\002\'\022%/fx/crosschain/v1/oracle_set/confirms'
  _QUERY.methods_by_name['LastOracleSetRequests']._options = None
  _QUERY.methods_by_name['LastOracleSetRequests']._serialized_options = b'\202\323\344\223\002\'\022%/fx/crosschain/v1/oracle_set/requests'
  _QUERY.methods_by_name['LastPendingOracleSetRequestByAddr']._options = None
  _QUERY.methods_by_name['LastPendingOracleSetRequestByAddr']._serialized_options = b'\202\323\344\223\002#\022!/fx/crosschain/v1/oracle_set/last'
  _QUERY.methods_by_name['LastPendingBatchRequestByAddr']._options = None
  _QUERY.methods_by_name['LastPendingBatchRequestByAddr']._serialized_options = b'\202\323\344\223\002\036\022\034/fx/crosschain/v1/batch/last'
  _QUERY.methods_by_name['LastEventNonceByAddr']._options = None
  _QUERY.methods_by_name['LastEventNonceByAddr']._serialized_options = b'\202\323\344\223\002&\022$/fx/crosschain/v1/oracle/event_nonce'
  _QUERY.methods_by_name['LastEventBlockHeightByAddr']._options = None
  _QUERY.methods_by_name['LastEventBlockHeightByAddr']._serialized_options = b'\202\323\344\223\002-\022+/fx/crosschain/v1/oracle/event/block_height'
  _QUERY.methods_by_name['BatchFees']._options = None
  _QUERY.methods_by_name['BatchFees']._serialized_options = b'\202\323\344\223\002\036\022\034/fx/crosschain/v1/batch_fees'
  _QUERY.methods_by_name['LastObservedBlockHeight']._options = None
  _QUERY.methods_by_name['LastObservedBlockHeight']._serialized_options = b'\202\323\344\223\002)\022\'/fx/crosschain/v1/observed/block_height'
  _QUERY.methods_by_name['OutgoingTxBatches']._options = None
  _QUERY.methods_by_name['OutgoingTxBatches']._serialized_options = b'\202\323\344\223\002%\022#/fx/crosschain/v1/batch/outgoing_tx'
  _QUERY.methods_by_name['BatchRequestByNonce']._options = None
  _QUERY.methods_by_name['BatchRequestByNonce']._serialized_options = b'\202\323\344\223\002!\022\037/fx/crosschain/v1/batch/request'
  _QUERY.methods_by_name['BatchConfirm']._options = None
  _QUERY.methods_by_name['BatchConfirm']._serialized_options = b'\202\323\344\223\002!\022\037/fx/crosschain/v1/batch/confirm'
  _QUERY.methods_by_name['BatchConfirms']._options = None
  _QUERY.methods_by_name['BatchConfirms']._serialized_options = b'\202\323\344\223\002\"\022 /fx/crosschain/v1/batch/confirms'
  _QUERY.methods_by_name['TokenToDenom']._options = None
  _QUERY.methods_by_name['TokenToDenom']._serialized_options = b'\202\323\344\223\002\031\022\027/fx/crosschain/v1/denom'
  _QUERY.methods_by_name['DenomToToken']._options = None
  _QUERY.methods_by_name['DenomToToken']._serialized_options = b'\202\323\344\223\002\031\022\027/fx/crosschain/v1/token'
  _QUERY.methods_by_name['GetOracleByAddr']._options = None
  _QUERY.methods_by_name['GetOracleByAddr']._serialized_options = b'\202\323\344\223\002\"\022 /fx/crosschain/v1/oracle_by_addr'
  _QUERY.methods_by_name['GetOracleByExternalAddr']._options = None
  _QUERY.methods_by_name['GetOracleByExternalAddr']._serialized_options = b'\202\323\344\223\002+\022)/fx/crosschain/v1/oracle_by_external_addr'
  _QUERY.methods_by_name['GetOracleByBridgerAddr']._options = None
  _QUERY.methods_by_name['GetOracleByBridgerAddr']._serialized_options = b'\202\323\344\223\002*\022(/fx/crosschain/v1/oracle_by_bridger_addr'
  _QUERY.methods_by_name['GetPendingSendToExternal']._options = None
  _QUERY.methods_by_name['GetPendingSendToExternal']._serialized_options = b'\202\323\344\223\002,\022*/fx/crosschain/v1/pending_send_to_external'
  _QUERY.methods_by_name['Oracles']._options = None
  _QUERY.methods_by_name['Oracles']._serialized_options = b'\202\323\344\223\002\033\022\031/fx/crosschain/v1/oracles'
  _QUERY.methods_by_name['ProjectedBatchTimeoutHeight']._options = None
  _QUERY.methods_by_name['ProjectedBatchTimeoutHeight']._serialized_options = b'\202\323\344\223\002+\022)/fx/crosschain/v1/projected_batch_timeout'
  _QUERY.methods_by_name['BridgeTokens']._options = None
  _QUERY.methods_by_name['BridgeTokens']._serialized_options = b'\202\323\344\223\002!\022\037/fx/crosschain/v1/bridge_tokens'
  _QUERY.methods_by_name['BridgeCoinByDenom']._options = None
  _QUERY.methods_by_name['BridgeCoinByDenom']._serialized_options = b'\202\323\344\223\002(\022&/fx/crosschain/v1/bridge_coin_by_denom'
  _QUERY.methods_by_name['BridgeChainList']._options = None
  _QUERY.methods_by_name['BridgeChainList']._serialized_options = b'\202\323\344\223\002%\022#/fx/crosschain/v1/bridge_chain_list'
  _globals['_QUERYPARAMSREQUEST']._serialized_start=199
  _globals['_QUERYPARAMSREQUEST']._serialized_end=239
  _globals['_QUERYPARAMSRESPONSE']._serialized_start=241
  _globals['_QUERYPARAMSRESPONSE']._serialized_end=318
  _globals['_QUERYCURRENTORACLESETREQUEST']._serialized_start=320
  _globals['_QUERYCURRENTORACLESETREQUEST']._serialized_end=370
  _globals['_QUERYCURRENTORACLESETRESPONSE']._serialized_start=372
  _globals['_QUERYCURRENTORACLESETRESPONSE']._serialized_end=460
  _globals['_QUERYORACLESETREQUESTREQUEST']._serialized_start=462
  _globals['_QUERYORACLESETREQUESTREQUEST']._serialized_end=527
  _globals['_QUERYORACLESETREQUESTRESPONSE']._serialized_start=529
  _globals['_QUERYORACLESETREQUESTRESPONSE']._serialized_end=617
  _globals['_QUERYORACLESETCONFIRMREQUEST']._serialized_start=619
  _globals['_QUERYORACLESETCONFIRMREQUEST']._serialized_end=709
  _globals['_QUERYORACLESETCONFIRMRESPONSE']._serialized_start=711
  _globals['_QUERYORACLESETCONFIRMRESPONSE']._serialized_end=806
  _globals['_QUERYORACLESETCONFIRMSBYNONCEREQUEST']._serialized_start=808
  _globals['_QUERYORACLESETCONFIRMSBYNONCEREQUEST']._serialized_end=881
  _globals['_QUERYORACLESETCONFIRMSBYNONCERESPONSE']._serialized_start=883
  _globals['_QUERYORACLESETCONFIRMSBYNONCERESPONSE']._serialized_end=987
  _globals['_QUERYLASTORACLESETREQUESTSREQUEST']._serialized_start=989
  _globals['_QUERYLASTORACLESETREQUESTSREQUEST']._serialized_end=1044
  _globals['_QUERYLASTORACLESETREQUESTSRESPONSE']._serialized_start=1046
  _globals['_QUERYLASTORACLESETREQUESTSRESPONSE']._serialized_end=1140
  _globals['_QUERYLASTPENDINGORACLESETREQUESTBYADDRREQUEST']._serialized_start=1142
  _globals['_QUERYLASTPENDINGORACLESETREQUESTBYADDRREQUEST']._serialized_end=1234
  _globals['_QUERYLASTPENDINGORACLESETREQUESTBYADDRRESPONSE']._serialized_start=1236
  _globals['_QUERYLASTPENDINGORACLESETREQUESTBYADDRRESPONSE']._serialized_end=1342
  _globals['_QUERYBATCHFEEREQUEST']._serialized_start=1344
  _globals['_QUERYBATCHFEEREQUEST']._serialized_end=1453
  _globals['_QUERYBATCHFEERESPONSE']._serialized_start=1455
  _globals['_QUERYBATCHFEERESPONSE']._serialized_end=1535
  _globals['_QUERYLASTPENDINGBATCHREQUESTBYADDRREQUEST']._serialized_start=1537
  _globals['_QUERYLASTPENDINGBATCHREQUESTBYADDRREQUEST']._serialized_end=1625
  _globals['_QUERYLASTPENDINGBATCHREQUESTBYADDRRESPONSE']._serialized_start=1627
  _globals['_QUERYLASTPENDINGBATCHREQUESTBYADDRRESPONSE']._serialized_end=1729
  _globals['_QUERYOUTGOINGTXBATCHESREQUEST']._serialized_start=1731
  _globals['_QUERYOUTGOINGTXBATCHESREQUEST']._serialized_end=1782
  _globals['_QUERYOUTGOINGTXBATCHESRESPONSE']._serialized_start=1784
  _globals['_QUERYOUTGOINGTXBATCHESRESPONSE']._serialized_end=1876
  _globals['_QUERYBATCHREQUESTBYNONCEREQUEST']._serialized_start=1878
  _globals['_QUERYBATCHREQUESTBYNONCEREQUEST']._serialized_end=1970
  _globals['_QUERYBATCHREQUESTBYNONCERESPONSE']._serialized_start=1972
  _globals['_QUERYBATCHREQUESTBYNONCERESPONSE']._serialized_end=2064
  _globals['_QUERYBATCHCONFIRMREQUEST']._serialized_start=2066
  _globals['_QUERYBATCHCONFIRMREQUEST']._serialized_end=2176
  _globals['_QUERYBATCHCONFIRMRESPONSE']._serialized_start=2178
  _globals['_QUERYBATCHCONFIRMRESPONSE']._serialized_end=2265
  _globals['_QUERYBATCHCONFIRMSREQUEST']._serialized_start=2267
  _globals['_QUERYBATCHCONFIRMSREQUEST']._serialized_end=2353
  _globals['_QUERYBATCHCONFIRMSRESPONSE']._serialized_start=2355
  _globals['_QUERYBATCHCONFIRMSRESPONSE']._serialized_end=2444
  _globals['_QUERYLASTEVENTNONCEBYADDRREQUEST']._serialized_start=2446
  _globals['_QUERYLASTEVENTNONCEBYADDRREQUEST']._serialized_end=2525
  _globals['_QUERYLASTEVENTNONCEBYADDRRESPONSE']._serialized_start=2527
  _globals['_QUERYLASTEVENTNONCEBYADDRRESPONSE']._serialized_end=2583
  _globals['_QUERYTOKENTODENOMREQUEST']._serialized_start=2585
  _globals['_QUERYTOKENTODENOMREQUEST']._serialized_end=2646
  _globals['_QUERYTOKENTODENOMRESPONSE']._serialized_start=2648
  _globals['_QUERYTOKENTODENOMRESPONSE']._serialized_end=2690
  _globals['_QUERYDENOMTOTOKENREQUEST']._serialized_start=2692
  _globals['_QUERYDENOMTOTOKENREQUEST']._serialized_end=2753
  _globals['_QUERYDENOMTOTOKENRESPONSE']._serialized_start=2755
  _globals['_QUERYDENOMTOTOKENRESPONSE']._serialized_end=2797
  _globals['_QUERYORACLEBYADDRREQUEST']._serialized_start=2799
  _globals['_QUERYORACLEBYADDRREQUEST']._serialized_end=2869
  _globals['_QUERYORACLERESPONSE']._serialized_start=2871
  _globals['_QUERYORACLERESPONSE']._serialized_end=2942
  _globals['_QUERYORACLEBYEXTERNALADDRREQUEST']._serialized_start=2944
  _globals['_QUERYORACLEBYEXTERNALADDRREQUEST']._serialized_end=3024
  _globals['_QUERYORACLEBYBRIDGERADDRREQUEST']._serialized_start=3026
  _globals['_QUERYORACLEBYBRIDGERADDRREQUEST']._serialized_end=3104
  _globals['_QUERYPENDINGSENDTOEXTERNALREQUEST']._serialized_start=3106
  _globals['_QUERYPENDINGSENDTOEXTERNALREQUEST']._serialized_end=3185
  _globals['_QUERYPENDINGSENDTOEXTERNALRESPONSE']._serialized_start=3188
  _globals['_QUERYPENDINGSENDTOEXTERNALRESPONSE']._serialized_end=3375
  _globals['_QUERYLASTOBSERVEDBLOCKHEIGHTREQUEST']._serialized_start=3377
  _globals['_QUERYLASTOBSERVEDBLOCKHEIGHTREQUEST']._serialized_end=3434
  _globals['_QUERYLASTOBSERVEDBLOCKHEIGHTRESPONSE']._serialized_start=3436
  _globals['_QUERYLASTOBSERVEDBLOCKHEIGHTRESPONSE']._serialized_end=3527
  _globals['_QUERYLASTEVENTBLOCKHEIGHTBYADDRREQUEST']._serialized_start=3529
  _globals['_QUERYLASTEVENTBLOCKHEIGHTBYADDRREQUEST']._serialized_end=3614
  _globals['_QUERYLASTEVENTBLOCKHEIGHTBYADDRRESPONSE']._serialized_start=3616
  _globals['_QUERYLASTEVENTBLOCKHEIGHTBYADDRRESPONSE']._serialized_end=3679
  _globals['_QUERYORACLESREQUEST']._serialized_start=3681
  _globals['_QUERYORACLESREQUEST']._serialized_end=3722
  _globals['_QUERYORACLESRESPONSE']._serialized_start=3724
  _globals['_QUERYORACLESRESPONSE']._serialized_end=3803
  _globals['_QUERYPROJECTEDBATCHTIMEOUTHEIGHTREQUEST']._serialized_start=3805
  _globals['_QUERYPROJECTEDBATCHTIMEOUTHEIGHTREQUEST']._serialized_end=3866
  _globals['_QUERYPROJECTEDBATCHTIMEOUTHEIGHTRESPONSE']._serialized_start=3868
  _globals['_QUERYPROJECTEDBATCHTIMEOUTHEIGHTRESPONSE']._serialized_end=3934
  _globals['_QUERYBRIDGETOKENSREQUEST']._serialized_start=3936
  _globals['_QUERYBRIDGETOKENSREQUEST']._serialized_end=3982
  _globals['_QUERYBRIDGETOKENSRESPONSE']._serialized_start=3984
  _globals['_QUERYBRIDGETOKENSRESPONSE']._serialized_end=4073
  _globals['_QUERYBRIDGECOINBYDENOMREQUEST']._serialized_start=4075
  _globals['_QUERYBRIDGECOINBYDENOMREQUEST']._serialized_end=4141
  _globals['_QUERYBRIDGECOINBYDENOMRESPONSE']._serialized_start=4143
  _globals['_QUERYBRIDGECOINBYDENOMRESPONSE']._serialized_end=4222
  _globals['_QUERYBRIDGECHAINLISTREQUEST']._serialized_start=4224
  _globals['_QUERYBRIDGECHAINLISTREQUEST']._serialized_end=4253
  _globals['_QUERYBRIDGECHAINLISTRESPONSE']._serialized_start=4255
  _globals['_QUERYBRIDGECHAINLISTRESPONSE']._serialized_end=4306
  _globals['_QUERY']._serialized_start=4309
  _globals['_QUERY']._serialized_end=9220
# @@protoc_insertion_point(module_scope)



import unittest

from google.protobuf.json_format import MessageToJson

from fxsdk.client.crosschain import CrossChainClient, CrossChainTarget
from fxsdk.msg import new_msg_send
from fxsdk.wallet.key import mnemonic_to_privkey
from fxsdk.wallet.builder import TxBuilder
from fxsdk.client.grpc import Client
from fxsdk.x.cosmos.base.v1beta1.coin_pb2 import Coin
from fxsdk.x.cosmos.tx.v1beta1.service_pb2 import BROADCAST_MODE_SYNC

grpc_url = 'localhost:9090'
address_prefix = 'mx'
private_key = mnemonic_to_privkey("test test test test test test test test test test test junk")
tx_builder = TxBuilder(private_key=private_key, prefix=address_prefix)


class TestSendTx(unittest.TestCase):

    def test_bank_send(self):
        from_addr = tx_builder.from_address().to_string()
        send_msg_any = new_msg_send(
            from_address=from_addr,
            to_address=from_addr,
            amount=[Coin(amount='100', denom='cusd')],
        )

        grpc_cli = Client(grpc_url)
        tx = grpc_cli.build_tx(tx_builder, [send_msg_any])
        print(MessageToJson(tx))

        tx_response = grpc_cli.broadcast_tx(tx, BROADCAST_MODE_SYNC)
        print(tx_response)
        self.assertEqual(tx_response.code, 0)

        tx_response = grpc_cli.wait_mint_tx(tx_response.txhash)
        print(tx_response)

    def test_ibc_transfer_mx2eth(self):
        grpc_cli = CrossChainClient(grpc_url)
        channels = grpc_cli.query_channels()
        if len(channels) == 0:
            print("no channel found")
            return
        channel = channels[0].channel_id
        to_address = tx_builder.from_address().to_hex()
        amount = Coin(denom='FX', amount='100')
        tx_response = grpc_cli.ibc_transfer(tx_builder=tx_builder, to_address=to_address, amount=amount,
                                            channel=channel, target=CrossChainTarget.Ethereum,
                                            mode=BROADCAST_MODE_SYNC)
        print(tx_response)
        self.assertEqual(tx_response.code, 0)

    def test_ibc_transfer_mx2mx(self):
        grpc_cli = CrossChainClient(grpc_url)
        channels = grpc_cli.query_channels()
        if len(channels) == 0:
            print("no channel found")
            return
        from_channel = channels[0].channel_id
        to_channel = "channel-6"
        amount = Coin(denom='FX', amount='100')
        tx_response = grpc_cli.ibc_transfer_mx2mx(tx_builder=tx_builder, amount=amount, from_channel=from_channel,
                                                  to_channel=to_channel, mode=BROADCAST_MODE_SYNC)
        print(tx_response)
        self.assertEqual(tx_response.code, 0)


if __name__ == '__main__':
    unittest.main()
